@@ -31,6 +31,14 @@ enum BVH_STAT {
 	BVH_STAT_TRIANGLE_COUNT,
 	BVH_STAT_CHILDNODE_COUNT,
 	BVH_STAT_QNODE_COUNT,
+	BVH_STAT_ALIGNED_COUNT,
+	BVH_STAT_UNALIGNED_COUNT,
+	BVH_STAT_ALIGNED_INNER_COUNT,
+	BVH_STAT_UNALIGNED_INNER_COUNT,
+	BVH_STAT_ALIGNED_INNER_QNODE_COUNT,
+	BVH_STAT_UNALIGNED_INNER_QNODE_COUNT,
+	BVH_STAT_ALIGNED_LEAF_COUNT,
+	BVH_STAT_UNALIGNED_LEAF_COUNT,
 };
 
 class BVHParams;
@@ -38,16 +46,41 @@ class BVHParams;
 class BVHNode
 {
 public:
-	BVHNode()
+	BVHNode() : m_is_unaligned(false),
+	            m_aligned_space(NULL)
 	{
 	}
 
-	virtual ~BVHNode() {}
+	virtual ~BVHNode()
+	{
+		delete m_aligned_space;
+	}
+
 	virtual bool is_leaf() const = 0;
 	virtual int num_children() const = 0;
 	virtual BVHNode *get_child(int i) const = 0;
 	virtual int num_triangles() const { return 0; }
 	virtual void print(int depth = 0) const = 0;
+	bool is_unaligned() const { return m_is_unaligned; }
+
+	inline void set_aligned_space(const Transform& aligned_space)
+	{
+		m_is_unaligned = true;
+		if(m_aligned_space == NULL) {
+			m_aligned_space = new Transform(aligned_space);
+		}
+		else {
+			*m_aligned_space = aligned_space;
+		}
+	}
+
+	inline Transform get_aligned_space() const
+	{
+		if(m_aligned_space == NULL) {
+			return transform_identity();
+		}
+		return *m_aligned_space;
+	}
 
 	BoundBox m_bounds;
 	uint m_visibility;
@@ -58,12 +91,20 @@ public:
 	void deleteSubtree();
 
 	uint update_visibility();
+
+	bool m_is_unaligned;
+
+	// TODO(sergey): Can be stored as 3x3 matrix, but better to have some
+	// utilities and type defines in util_transform first.
+	Transform *m_aligned_space;
 };
 
 class InnerNode : public BVHNode
 {
 public:
-	InnerNode(const BoundBox& bounds, BVHNode* child0, BVHNode* child1)
+	InnerNode(const BoundBox& bounds,
+	          BVHNode* child0,
+	          BVHNode* child1)
 	{
 		m_bounds = bounds;
 		children[0] = child0;
@@ -75,7 +116,7 @@ public:
 			m_visibility = 0; /* happens on build cancel */
 	}
 
-	InnerNode(const BoundBox& bounds)
+	explicit InnerNode(const BoundBox& bounds)
 	{
 		m_bounds = bounds;
 		m_visibility = 0;
