@@ -49,13 +49,21 @@ std::ostream& operator <<(std::ostream &os,
 	/* TODO(sergey): Decode bitflag into list of names. */
 	os << "Nodes features: " << requested_features.nodes_features << std::endl;
 	os << "Use hair: "
-	   << string_from_bool(requested_features.use_hair)  << std::endl;
+	   << string_from_bool(requested_features.use_hair) << std::endl;
 	os << "Use object motion: "
-	   << string_from_bool(requested_features.use_object_motion)  << std::endl;
+	   << string_from_bool(requested_features.use_object_motion) << std::endl;
 	os << "Use camera motion: "
-	   << string_from_bool(requested_features.use_camera_motion)  << std::endl;
+	   << string_from_bool(requested_features.use_camera_motion) << std::endl;
 	os << "Use Baking: "
-	   << string_from_bool(requested_features.use_baking)  << std::endl;
+	   << string_from_bool(requested_features.use_baking) << std::endl;
+	os << "Use Subsurface: "
+	   << string_from_bool(requested_features.use_subsurface) << std::endl;
+	os << "Use Volume: "
+	   << string_from_bool(requested_features.use_volume) << std::endl;
+	os << "Use Branched Integrator: "
+	   << string_from_bool(requested_features.use_integrator_branched) << std::endl;
+	os << "Use Patch Evaluation: "
+	   << string_from_bool(requested_features.use_patch_evaluation) << std::endl;
 	return os;
 }
 
