@@ -63,12 +63,12 @@ public:
 
 	void set_shader_limit(const size_t x, const size_t y);
 
-	bool bake(Device *device, DeviceScene *dscene, Scene *scene, Progress& progress, ShaderEvalType shader_type, BakeData *bake_data, float result[]);
+	bool bake(Device *device, DeviceScene *dscene, Scene *scene, Progress& progress, ShaderEvalType shader_type, const int pass_filter, BakeData *bake_data, float result[]);
 
 	void device_update(Device *device, DeviceScene *dscene, Scene *scene, Progress& progress);
 	void device_free(Device *device, DeviceScene *dscene);
 
-	static bool is_light_pass(ShaderEvalType type);
+	static int shader_type_to_pass_filter(ShaderEvalType type, const int pass_filter);
 	static bool is_aa_pass(ShaderEvalType type);
 
 	bool need_update;
