@@ -100,38 +100,53 @@ static void shade_background_pixels(Device *device, DeviceScene *dscene, int res
 
 /* Light */
 
-Light::Light()
+NODE_DEFINE(Light)
 {
-	type = LIGHT_POINT;
+	NodeType* type = NodeType::add("light", create);
 
-	co = make_float3(0.0f, 0.0f, 0.0f);
+	static NodeEnum type_enum;
+	type_enum.insert("point", LIGHT_POINT);
+	type_enum.insert("background", LIGHT_BACKGROUND);
+	type_enum.insert("area", LIGHT_AREA);
+	type_enum.insert("spot", LIGHT_SPOT);
+	SOCKET_ENUM(type, "Type", type_enum, LIGHT_POINT);
 
-	dir = make_float3(0.0f, 0.0f, 0.0f);
-	size = 0.0f;
+	SOCKET_POINT(co, "Co", make_float3(0.0f, 0.0f, 0.0f));
 
-	axisu = make_float3(0.0f, 0.0f, 0.0f);
-	sizeu = 1.0f;
-	axisv = make_float3(0.0f, 0.0f, 0.0f);
-	sizev = 1.0f;
+	SOCKET_VECTOR(dir, "Dir", make_float3(0.0f, 0.0f, 0.0f));
+	SOCKET_FLOAT(size, "Size", 0.0f);
 
-	map_resolution = 512;
+	SOCKET_VECTOR(axisu, "Axis U", make_float3(0.0f, 0.0f, 0.0f));
+	SOCKET_FLOAT(sizeu, "Size U", 1.0f);
+	SOCKET_VECTOR(axisv, "Axis V", make_float3(0.0f, 0.0f, 0.0f));
+	SOCKET_FLOAT(sizev, "Size V", 1.0f);
 
-	spot_angle = M_PI_4_F;
-	spot_smooth = 0.0f;
+	SOCKET_INT(map_resolution, "Map Resolution", 512);
 
-	cast_shadow = true;
-	use_mis = false;
-	use_diffuse = true;
-	use_glossy = true;
-	use_transmission = true;
-	use_scatter = true;
+	SOCKET_FLOAT(spot_angle, "Spot Angle", M_PI_4_F);
+	SOCKET_FLOAT(spot_smooth, "Spot Smooth", 0.0f);
 
-	shader = 0;
-	samples = 1;
-	max_bounces = 1024;
+	SOCKET_BOOLEAN(cast_shadow, "Cast Shadow", true);
+	SOCKET_BOOLEAN(use_mis, "Use Mis", false);
+	SOCKET_BOOLEAN(use_diffuse, "Use Diffuse", true);
+	SOCKET_BOOLEAN(use_glossy, "Use Glossy", true);
+	SOCKET_BOOLEAN(use_transmission, "Use Transmission", true);
+	SOCKET_BOOLEAN(use_scatter, "Use Scatter", true);
 
-	is_portal = false;
-	is_enabled = true;
+	SOCKET_INT(samples, "Samples", 1);
+	SOCKET_INT(max_bounces, "Max Bounces", 1024);
+
+	SOCKET_BOOLEAN(is_portal, "Is Portal", false);
+	SOCKET_BOOLEAN(is_enabled, "Is Enabled", true);
+
+	SOCKET_NODE(shader, "Shader", &Shader::node_type);
+
+	return type;
+}
+
+Light::Light()
+: Node(node_type)
+{
 }
 
 void Light::tag_update(Scene *scene)
@@ -147,7 +162,7 @@ bool Light::has_contribution(Scene *scene)
 	if(type == LIGHT_BACKGROUND) {
 		return true;
 	}
-	return scene->shaders[shader]->has_surface_emission;
+	return (shader) ? shader->has_surface_emission : scene->default_light->has_surface_emission;
 }
 
 /* Light Manager */
@@ -162,6 +177,16 @@ LightManager::~LightManager()
 {
 }
 
+bool LightManager::has_background_light(Scene *scene)
+{
+	foreach(Light *light, scene->lights) {
+		if(light->type == LIGHT_BACKGROUND) {
+			return true;
+		}
+	}
+	return false;
+}
+
 void LightManager::disable_ineffective_light(Device *device, Scene *scene)
 {
 	/* Make all lights enabled by default, and perform some preliminary checks
@@ -180,7 +205,7 @@ void LightManager::disable_ineffective_light(Device *device, Scene *scene)
 		 * - If unsupported on a device
 		 * - If we don't need it (no HDRs etc.)
 		 */
-		Shader *shader = scene->shaders[scene->background->shader];
+		Shader *shader = (scene->background->shader) ? scene->background->shader : scene->default_background;
 		bool disable_mis = !(has_portal || shader->has_surface_spatial_varying) ||
 		                   !(device->info.advanced_shading);
 		if(disable_mis) {
@@ -194,56 +219,73 @@ void LightManager::disable_ineffective_light(Device *device, Scene *scene)
 	}
 }
 
+bool LightManager::object_usable_as_light(Object *object) {
+	Mesh *mesh = object->mesh;
+	/* Skip if we are not visible for BSDFs. */
+	if(!(object->visibility & (PATH_RAY_DIFFUSE|PATH_RAY_GLOSSY|PATH_RAY_TRANSMIT))) {
+		return false;
+	}
+	/* Skip motion blurred deforming meshes, not supported yet. */
+	if(mesh->has_motion_blur()) {
+		return false;
+	}
+	/* Skip if we have no emission shaders. */
+	/* TODO(sergey): Ideally we want to avoid such duplicated loop, since it'll
+	 * iterate all mesh shaders twice (when counting and when calculating
+	 * triangle area.
+	 */
+	foreach(const Shader *shader, mesh->used_shaders) {
+		if(shader->use_mis && shader->has_surface_emission) {
+			return true;
+		}
+	}
+	return false;
+}
+
 void LightManager::device_update_distribution(Device *device, DeviceScene *dscene, Scene *scene, Progress& progress)
 {
 	progress.set_status("Updating Lights", "Computing distribution");
 
 	/* count */
 	size_t num_lights = 0;
+	size_t num_portals = 0;
 	size_t num_background_lights = 0;
 	size_t num_triangles = 0;
 
 	bool background_mis = false;
 
 	foreach(Light *light, scene->lights) {
-		if(light->is_enabled)
+		if(light->is_enabled) {
 			num_lights++;
+		}
+		if(light->is_portal) {
+			num_portals++;
+		}
 	}
 
 	foreach(Object *object, scene->objects) {
+		if(progress.get_cancel()) return;
+
+		if(!object_usable_as_light(object)) {
+			continue;
+		}
+		/* Count triangles. */
 		Mesh *mesh = object->mesh;
-		bool have_emission = false;
-
-		/* skip if we are not visible for BSDFs */
-		if(!(object->visibility & (PATH_RAY_DIFFUSE|PATH_RAY_GLOSSY|PATH_RAY_TRANSMIT)))
-			continue;
-
-		/* skip motion blurred deforming meshes, not supported yet */
-		if(mesh->has_motion_blur())
-			continue;
-
-		/* skip if we have no emission shaders */
-		foreach(uint sindex, mesh->used_shaders) {
-			Shader *shader = scene->shaders[sindex];
+		size_t mesh_num_triangles = mesh->num_triangles();
+		for(size_t i = 0; i < mesh_num_triangles; i++) {
+			int shader_index = mesh->shader[i];
+			Shader *shader = (shader_index < mesh->used_shaders.size())
+			                         ? mesh->used_shaders[shader_index]
+			                         : scene->default_surface;
 
 			if(shader->use_mis && shader->has_surface_emission) {
-				have_emission = true;
-				break;
-			}
-		}
-
-		/* count triangles */
-		if(have_emission) {
-			for(size_t i = 0; i < mesh->triangles.size(); i++) {
-				Shader *shader = scene->shaders[mesh->shader[i]];
-
-				if(shader->use_mis && shader->has_surface_emission)
-					num_triangles++;
+				num_triangles++;
 			}
 		}
 	}
 
 	size_t num_distribution = num_triangles + num_lights;
+	VLOG(1) << "Total " << num_distribution << " of light distribution primitives.";
 
 	/* emission area */
 	float4 *distribution = dscene->light_distribution.resize(num_distribution + 1);
@@ -254,86 +296,65 @@ void LightManager::device_update_distribution(Device *device, DeviceScene *dscen
 	int j = 0;
 
 	foreach(Object *object, scene->objects) {
+		if(progress.get_cancel()) return;
+
+		if(!object_usable_as_light(object)) {
+			j++;
+			continue;
+		}
+		/* Sum area. */
 		Mesh *mesh = object->mesh;
-		bool have_emission = false;
+		bool transform_applied = mesh->transform_applied;
+		Transform tfm = object->tfm;
+		int object_id = j;
+		int shader_flag = 0;
 
-		/* skip if we are not visible for BSDFs */
-		if(!(object->visibility & (PATH_RAY_DIFFUSE|PATH_RAY_GLOSSY|PATH_RAY_TRANSMIT))) {
-			j++;
-			continue;
+		if(!(object->visibility & PATH_RAY_DIFFUSE)) {
+			shader_flag |= SHADER_EXCLUDE_DIFFUSE;
+			use_light_visibility = true;
+		}
+		if(!(object->visibility & PATH_RAY_GLOSSY)) {
+			shader_flag |= SHADER_EXCLUDE_GLOSSY;
+			use_light_visibility = true;
+		}
+		if(!(object->visibility & PATH_RAY_TRANSMIT)) {
+			shader_flag |= SHADER_EXCLUDE_TRANSMIT;
+			use_light_visibility = true;
+		}
+		if(!(object->visibility & PATH_RAY_VOLUME_SCATTER)) {
+			shader_flag |= SHADER_EXCLUDE_SCATTER;
+			use_light_visibility = true;
 		}
 
-		/* skip motion blurred deforming meshes, not supported yet */
-		if(mesh->has_motion_blur()) {
-			j++;
-			continue;
-		}
-
-		/* skip if we have no emission shaders */
-		foreach(uint sindex, mesh->used_shaders) {
-			Shader *shader = scene->shaders[sindex];
+		size_t mesh_num_triangles = mesh->num_triangles();
+		for(size_t i = 0; i < mesh_num_triangles; i++) {
+			int shader_index = mesh->shader[i];
+			Shader *shader = (shader_index < mesh->used_shaders.size())
+			                         ? mesh->used_shaders[shader_index]
+			                         : scene->default_surface;
 
 			if(shader->use_mis && shader->has_surface_emission) {
-				have_emission = true;
-				break;
-			}
-		}
+				distribution[offset].x = totarea;
+				distribution[offset].y = __int_as_float(i + mesh->tri_offset);
+				distribution[offset].z = __int_as_float(shader_flag);
+				distribution[offset].w = __int_as_float(object_id);
+				offset++;
 
-		/* sum area */
-		if(have_emission) {
-			bool transform_applied = mesh->transform_applied;
-			Transform tfm = object->tfm;
-			int object_id = j;
-			int shader_flag = 0;
+				Mesh::Triangle t = mesh->get_triangle(i);
+				float3 p1 = mesh->verts[t.v[0]];
+				float3 p2 = mesh->verts[t.v[1]];
+				float3 p3 = mesh->verts[t.v[2]];
 
-			if(transform_applied)
-				object_id = ~object_id;
-
-			if(!(object->visibility & PATH_RAY_DIFFUSE)) {
-				shader_flag |= SHADER_EXCLUDE_DIFFUSE;
-				use_light_visibility = true;
-			}
-			if(!(object->visibility & PATH_RAY_GLOSSY)) {
-				shader_flag |= SHADER_EXCLUDE_GLOSSY;
-				use_light_visibility = true;
-			}
-			if(!(object->visibility & PATH_RAY_TRANSMIT)) {
-				shader_flag |= SHADER_EXCLUDE_TRANSMIT;
-				use_light_visibility = true;
-			}
-			if(!(object->visibility & PATH_RAY_VOLUME_SCATTER)) {
-				shader_flag |= SHADER_EXCLUDE_SCATTER;
-				use_light_visibility = true;
-			}
-
-			for(size_t i = 0; i < mesh->triangles.size(); i++) {
-				Shader *shader = scene->shaders[mesh->shader[i]];
-
-				if(shader->use_mis && shader->has_surface_emission) {
-					distribution[offset].x = totarea;
-					distribution[offset].y = __int_as_float(i + mesh->tri_offset);
-					distribution[offset].z = __int_as_float(shader_flag);
-					distribution[offset].w = __int_as_float(object_id);
-					offset++;
-
-					Mesh::Triangle t = mesh->triangles[i];
-					float3 p1 = mesh->verts[t.v[0]];
-					float3 p2 = mesh->verts[t.v[1]];
-					float3 p3 = mesh->verts[t.v[2]];
-
-					if(!transform_applied) {
-						p1 = transform_point(&tfm, p1);
-						p2 = transform_point(&tfm, p2);
-						p3 = transform_point(&tfm, p3);
-					}
-
-					totarea += triangle_area(p1, p2, p3);
+				if(!transform_applied) {
+					p1 = transform_point(&tfm, p1);
+					p2 = transform_point(&tfm, p2);
+					p3 = transform_point(&tfm, p3);
 				}
+
+				totarea += triangle_area(p1, p2, p3);
 			}
 		}
 
-		if(progress.get_cancel()) return;
-
 		j++;
 	}
 
@@ -426,9 +447,9 @@ void LightManager::device_update_distribution(Device *device, DeviceScene *dscen
 		device->tex_alloc("__light_distribution", dscene->light_distribution);
 
 		/* Portals */
-		if(num_background_lights > 0 && light_index != scene->lights.size()) {
+		if(num_portals > 0) {
 			kintegrator->portal_offset = light_index;
-			kintegrator->num_portals = scene->lights.size() - light_index;
+			kintegrator->num_portals = num_portals;
 			kintegrator->portal_pdf = background_mis? 0.5f: 1.0f;
 		}
 		else {
@@ -592,10 +613,21 @@ void LightManager::device_update_points(Device *device,
                                         Scene *scene)
 {
 	int num_scene_lights = scene->lights.size();
-	if(num_scene_lights == 0)
-		return;
 
-	float4 *light_data = dscene->light_data.resize(num_scene_lights*LIGHT_SIZE);
+	int num_lights = 0;
+	foreach(Light *light, scene->lights) {
+		if(light->is_enabled || light->is_portal) {
+			num_lights++;
+		}
+	}
+
+	float4 *light_data = dscene->light_data.resize(num_lights*LIGHT_SIZE);
+
+	if(num_lights == 0) {
+		VLOG(1) << "No effective light, ignoring points update.";
+		return;
+	}
+
 	int light_index = 0;
 
 	foreach(Light *light, scene->lights) {
@@ -604,7 +636,8 @@ void LightManager::device_update_points(Device *device,
 		}
 
 		float3 co = light->co;
-		int shader_id = scene->shader_manager->get_shader_id(light->shader);
+		Shader *shader = (light->shader) ? light->shader : scene->default_light;
+		int shader_id = scene->shader_manager->get_shader_id(shader);
 		float samples = __int_as_float(light->samples);
 		float max_bounces = __int_as_float(light->max_bounces);
 
@@ -771,11 +804,11 @@ void LightManager::device_update_points(Device *device,
 
 void LightManager::device_update(Device *device, DeviceScene *dscene, Scene *scene, Progress& progress)
 {
-	VLOG(1) << "Total " << scene->lights.size() << " lights.";
-
 	if(!need_update)
 		return;
 
+	VLOG(1) << "Total " << scene->lights.size() << " lights.";
+
 	device_free(device, dscene);
 
 	use_light_visibility = false;
