@@ -19,6 +19,8 @@
 
 #include "kernel_types.h"
 
+#include "node.h"
+
 #include "util_types.h"
 #include "util_vector.h"
 
@@ -26,11 +28,15 @@ CCL_NAMESPACE_BEGIN
 
 class Device;
 class DeviceScene;
+class Object;
 class Progress;
 class Scene;
+class Shader;
 
-class Light {
+class Light : public Node {
 public:
+	NODE_DECLARE;
+
 	Light();
 
 	LightType type;
@@ -59,7 +65,7 @@ public:
 	bool is_portal;
 	bool is_enabled;
 
-	int shader;
+	Shader *shader;
 	int samples;
 	int max_bounces;
 
@@ -85,6 +91,9 @@ public:
 
 	void tag_update(Scene *scene);
 
+	/* Check whether there is a background light. */
+	bool has_background_light(Scene *scene);
+
 protected:
 	/* Optimization: disable light which is either unsupported or
 	 * which doesn't contribute to the scene or which is only used for MIS
@@ -103,6 +112,9 @@ protected:
 	                              DeviceScene *dscene,
 	                              Scene *scene,
 	                              Progress& progress);
+
+	/* Check whether light manager can use the object as a light-emissive. */
+	bool object_usable_as_light(Object *object);
 };
 
 CCL_NAMESPACE_END
