@@ -45,7 +45,7 @@ public:
 	void device_free(Device *device, DeviceScene *dscene);
 
 	size_t add_table(DeviceScene *dscene, vector<float>& data);
-	void remove_table(size_t offset);
+	void remove_table(size_t *offset);
 };
 
 CCL_NAMESPACE_END
