@@ -41,51 +41,65 @@ EdgeDice::EdgeDice(const SubdParams& params_)
 	}
 }
 
-void EdgeDice::reserve(int num_verts, int num_tris)
+void EdgeDice::reserve(int num_verts)
 {
 	Mesh *mesh = params.mesh;
 
 	vert_offset = mesh->verts.size();
-	tri_offset = mesh->triangles.size();
+	tri_offset = mesh->num_triangles();
 
-	mesh->reserve(vert_offset + num_verts, tri_offset + num_tris, 0, 0);
+	/* todo: optimize so we can reserve in advance, this is like push_back_slow() */
+	if(vert_offset + num_verts > mesh->verts.capacity()) {
+		mesh->reserve_mesh(size_t((vert_offset + num_verts) * 1.2), mesh->num_triangles());
+	}
+
+	mesh->resize_mesh(vert_offset + num_verts, tri_offset);
 
 	Attribute *attr_vN = mesh->attributes.add(ATTR_STD_VERTEX_NORMAL);
 
-	mesh_P = &mesh->verts[0];
+	mesh_P = mesh->verts.data();
 	mesh_N = attr_vN->data_float3();
 }
 
 int EdgeDice::add_vert(Patch *patch, float2 uv)
 {
-	float3 P, N, dPdu, dPdv;
+	float3 P, N;
 
-	patch->eval(&P, &dPdu, &dPdv, uv.x, uv.y);
-	N = normalize(cross(dPdu, dPdv));
+	patch->eval(&P, NULL, NULL, &N, uv.x, uv.y);
 
 	assert(vert_offset < params.mesh->verts.size());
 
 	mesh_P[vert_offset] = P;
 	mesh_N[vert_offset] = N;
+	params.mesh->vert_patch_uv[vert_offset] = make_float2(uv.x, uv.y);
 
 	if(params.ptex) {
 		Attribute *attr_ptex_uv = params.mesh->attributes.add(ATTR_STD_PTEX_UV);
-		params.mesh->attributes.reserve();
+		params.mesh->attributes.resize();
 
 		float3 *ptex_uv = attr_ptex_uv->data_float3();
 		ptex_uv[vert_offset] = make_float3(uv.x, uv.y, 0.0f);
 	}
 
+	params.mesh->num_subd_verts++;
+
 	return vert_offset++;
 }
 
 void EdgeDice::add_triangle(Patch *patch, int v0, int v1, int v2)
 {
-	params.mesh->add_triangle(v0, v1, v2, params.shader, params.smooth);
+	Mesh *mesh = params.mesh;
+
+	/* todo: optimize so we can reserve in advance, this is like push_back_slow() */
+	if(mesh->triangles.size() == mesh->triangles.capacity())
+		mesh->reserve_mesh(mesh->verts.size(), size_t(max(mesh->num_triangles() + 1, 1) * 1.2));
+
+	mesh->add_triangle(v0, v1, v2, patch->shader, true);
+	params.mesh->triangle_patch[params.mesh->num_triangles()-1] = patch->patch_index;
 
 	if(params.ptex) {
 		Attribute *attr_ptex_face_id = params.mesh->attributes.add(ATTR_STD_PTEX_FACE_ID);
-		params.mesh->attributes.reserve();
+		params.mesh->attributes.resize();
 
 		float *ptex_face_id = attr_ptex_face_id->data_float();
 		ptex_face_id[tri_offset] = (float)patch->ptex_face_id();
@@ -142,8 +156,7 @@ void QuadDice::reserve(EdgeFactors& ef, int Mu, int Mv)
 {
 	/* XXX need to make this also work for edge factor 0 and 1 */
 	int num_verts = (ef.tu0 + ef.tu1 + ef.tv0 + ef.tv1) + (Mu - 1)*(Mv - 1);
-	int num_tris = 0;
-	EdgeDice::reserve(num_verts, num_tris);
+	EdgeDice::reserve(num_verts);
 }
 
 float2 QuadDice::map_uv(SubPatch& sub, float u, float v)
@@ -159,7 +172,7 @@ float3 QuadDice::eval_projected(SubPatch& sub, float u, float v)
 	float2 uv = map_uv(sub, u, v);
 	float3 P;
 
-	sub.patch->eval(&P, NULL, NULL, uv.x, uv.y);
+	sub.patch->eval(&P, NULL, NULL, NULL, uv.x, uv.y);
 	if(params.camera)
 		P = transform_perspective(&params.camera->worldtoraster, P);
 
@@ -260,17 +273,10 @@ float QuadDice::scale_factor(SubPatch& sub, EdgeFactors& ef, int Mu, int Mv)
 void QuadDice::add_corners(SubPatch& sub)
 {
 	/* add verts for patch corners */
-	if(sub.patch->is_triangle()) {
-		add_vert(sub, 0.0f, 0.0f);
-		add_vert(sub, 1.0f, 0.0f);
-		add_vert(sub, 0.0f, 1.0f);
-	}
-	else {
-		add_vert(sub, 0.0f, 0.0f);
-		add_vert(sub, 1.0f, 0.0f);
-		add_vert(sub, 0.0f, 1.0f);
-		add_vert(sub, 1.0f, 1.0f);
-	}
+	add_vert(sub, 0.0f, 0.0f);
+	add_vert(sub, 1.0f, 0.0f);
+	add_vert(sub, 0.0f, 1.0f);
+	add_vert(sub, 1.0f, 1.0f);
 }
 
 void QuadDice::add_grid(SubPatch& sub, int Mu, int Mv, int offset)
@@ -305,7 +311,12 @@ void QuadDice::dice(SubPatch& sub, EdgeFactors& ef)
 	int Mu = max(ef.tu0, ef.tu1);
 	int Mv = max(ef.tv0, ef.tv1);
 
+#if 0 /* Doesnt work very well, especially at grazing angles. */
 	float S = scale_factor(sub, ef, Mu, Mv);
+#else
+	float S = 1.0f;
+#endif
+
 	Mu = max((int)ceil(S*Mu), 2); // XXX handle 0 & 1?
 	Mv = max((int)ceil(S*Mv), 2); // XXX handle 0 & 1?
 
@@ -338,144 +349,5 @@ void QuadDice::dice(SubPatch& sub, EdgeFactors& ef)
 	assert(vert_offset == params.mesh->verts.size());
 }
 
-/* TriangleDice */
-
-TriangleDice::TriangleDice(const SubdParams& params_)
-: EdgeDice(params_)
-{
-}
-
-void TriangleDice::reserve(EdgeFactors& ef, int M)
-{
-	int num_verts = ef.tu + ef.tv + ef.tw;
-
-	for(int m = M-2; m > 0; m -= 2)
-		num_verts += 3 + (m-1)*3;
-	
-	if(!(M & 1))
-		num_verts++;
-	
-	EdgeDice::reserve(num_verts, 0);
-}
-
-float2 TriangleDice::map_uv(SubPatch& sub, float2 uv)
-{
-	/* map UV from subpatch to patch parametric coordinates */
-	return uv.x*sub.Pu + uv.y*sub.Pv + (1.0f - uv.x - uv.y)*sub.Pw;
-}
-
-int TriangleDice::add_vert(SubPatch& sub, float2 uv)
-{
-	return EdgeDice::add_vert(sub.patch, map_uv(sub, uv));
-}
-
-void TriangleDice::add_grid(SubPatch& sub, EdgeFactors& ef, int M)
-{
-	// XXX normals are flipped, why?
-
-	/* grid is constructed starting from the outside edges, and adding
-	 * progressively smaller inner triangles that connected to the outer
-	 * one, until M = 1 or 2, the we fill up the last part. */
-	vector<int> outer_u, outer_v, outer_w;
-	int m;
-
-	/* add outer corners vertices */
-	{
-		float2 p_u = make_float2(1.0f, 0.0f);
-		float2 p_v = make_float2(0.0f, 1.0f);
-		float2 p_w = make_float2(0.0f, 0.0f);
-
-		int corner_u = add_vert(sub, p_u);
-		int corner_v = add_vert(sub, p_v);
-		int corner_w = add_vert(sub, p_w);
-
-		outer_u.push_back(corner_v);
-		outer_v.push_back(corner_w);
-		outer_w.push_back(corner_u);
-
-		for(int i = 1; i < ef.tu; i++)
-			outer_u.push_back(add_vert(sub, interp(p_v, p_w, i/(float)ef.tu)));
-		for(int i = 1; i < ef.tv; i++)
-			outer_v.push_back(add_vert(sub, interp(p_w, p_u, i/(float)ef.tv)));
-		for(int i = 1; i < ef.tw; i++)
-			outer_w.push_back(add_vert(sub, interp(p_u, p_v, i/(float)ef.tw)));
-
-		outer_u.push_back(corner_w);
-		outer_v.push_back(corner_u);
-		outer_w.push_back(corner_v);
-	}
-
-	for(m = M-2; m > 0; m -= 2) {
-		vector<int> inner_u, inner_v, inner_w;
-
-		const float t0 = m / (float)M;
-		float2 center = make_float2(1.0f/3.0f, 1.0f/3.0f);
-
-		/* 3 corner vertices */
-		float2 p_u = interp(center, make_float2(1.0f, 0.0f), t0);
-		float2 p_v = interp(center, make_float2(0.0f, 1.0f), t0);
-		float2 p_w = interp(center, make_float2(0.0f, 0.0f), t0);
-
-		int corner_u = add_vert(sub, p_u);
-		int corner_v = add_vert(sub, p_v);
-		int corner_w = add_vert(sub, p_w);
-
-		/* construct array of vertex indices for each side */
-		inner_u.push_back(corner_v);
-		inner_v.push_back(corner_w);
-		inner_w.push_back(corner_u);
-
-		for(int i = 1; i < m; i++) {
-			/* add vertices between corners */
-			const float t1 = i / (float)m;
-
-			inner_u.push_back(add_vert(sub, interp(p_v, p_w, t1)));
-			inner_v.push_back(add_vert(sub, interp(p_w, p_u, t1)));
-			inner_w.push_back(add_vert(sub, interp(p_u, p_v, t1)));
-		}
-
-		inner_u.push_back(corner_w);
-		inner_v.push_back(corner_u);
-		inner_w.push_back(corner_v);
-
-		/* stitch together inner/outer with triangles */
-		stitch_triangles(sub.patch, outer_u, inner_u);
-		stitch_triangles(sub.patch, outer_v, inner_v);
-		stitch_triangles(sub.patch, outer_w, inner_w);
-
-		outer_u = inner_u;
-		outer_v = inner_v;
-		outer_w = inner_w;
-	}
-
-	/* fill up last part */
-	if(m == -1) {
-		/* single triangle */
-		add_triangle(sub.patch, outer_w[0], outer_u[0], outer_v[0]);
-	}
-	else {
-		/* center vertex + 6 triangles */
-		int center = add_vert(sub, make_float2(1.0f/3.0f, 1.0f/3.0f));
-
-		add_triangle(sub.patch, outer_w[0], outer_w[1], center);
-		add_triangle(sub.patch, outer_w[1], outer_w[2], center);
-		add_triangle(sub.patch, outer_u[0], outer_u[1], center);
-		add_triangle(sub.patch, outer_u[1], outer_u[2], center);
-		add_triangle(sub.patch, outer_v[0], outer_v[1], center);
-		add_triangle(sub.patch, outer_v[1], outer_v[2], center);
-	}
-}
-
-void TriangleDice::dice(SubPatch& sub, EdgeFactors& ef)
-{
-	/* todo: handle 2 1 1 resolution */
-	int M = max(ef.tu, max(ef.tv, ef.tw));
-
-	reserve(ef, M);
-	add_grid(sub, ef, M);
-
-	assert(vert_offset == params.mesh->verts.size());
-}
-
 CCL_NAMESPACE_END
 
