@@ -57,7 +57,6 @@ typedef struct _stat path_stat_t;
 #  ifndef S_ISDIR
 #    define S_ISDIR(x) (((x) & _S_IFDIR) == _S_IFDIR)
 #  endif
-#  define mkdir(path, mode) _mkdir(path)
 #else
 typedef struct stat path_stat_t;
 #endif
@@ -93,7 +92,7 @@ public:
 	{
 	}
 
-	directory_iterator(const string& path)
+	explicit directory_iterator(const string& path)
 	: path_(path),
 	  path_info_(path, find_data_)
 	{
@@ -177,7 +176,7 @@ class directory_iterator {
 public:
 	class path_info {
 	public:
-		path_info(const string& path)
+		explicit path_info(const string& path)
 		: path_(path),
 		  entry_(NULL)
 		{
@@ -204,7 +203,7 @@ public:
 	{
 	}
 
-	directory_iterator(const string& path)
+	explicit directory_iterator(const string& path)
 	: path_(path),
 	  path_info_(path_),
 	  cur_entry_(0)
@@ -486,9 +485,9 @@ static string path_unc_to_short(const string& path)
 		if((len > 5) && (path[5] ==  ':')) {
 			return path.substr(4, len - 4);
 		}
-		else if ((len > 7) &&
-		         (path.substr(4, 3) == "UNC") &&
-		         ((path[7] ==  DIR_SEP) || (path[7] ==  DIR_SEP_ALT)))
+		else if((len > 7) &&
+		        (path.substr(4, 3) == "UNC") &&
+		        ((path[7] ==  DIR_SEP) || (path[7] ==  DIR_SEP_ALT)))
 		{
 			return "\\\\" + path.substr(8, len - 8);
 		}
@@ -634,7 +633,12 @@ static bool create_directories_recursivey(const string& path)
 		}
 	}
 
+#ifdef _WIN32
+	wstring path_wc = string_to_wstring(path);
+	return _wmkdir(path_wc.c_str()) == 0;
+#else
 	return mkdir(path.c_str(), 0777) == 0;
+#endif
 }
 
 void path_create_directories(const string& filepath)
@@ -724,36 +728,73 @@ bool path_remove(const string& path)
 	return remove(path.c_str()) == 0;
 }
 
-string path_source_replace_includes(const string& source_, const string& path)
+static string line_directive(const string& path, int line)
 {
-	/* our own little c preprocessor that replaces #includes with the file
+	string escaped_path = path;
+	string_replace(escaped_path, "\"", "\\\"");
+	string_replace(escaped_path, "\'", "\\\'");
+	string_replace(escaped_path, "\?", "\\\?");
+	string_replace(escaped_path, "\\", "\\\\");
+	return string_printf("#line %d \"%s\"", line, escaped_path.c_str());
+}
+
+
+string path_source_replace_includes(const string& source,
+                                    const string& path,
+                                    const string& source_filename)
+{
+	/* Our own little c preprocessor that replaces #includes with the file
 	 * contents, to work around issue of opencl drivers not supporting
-	 * include paths with spaces in them */
-	string source = source_;
-	const string include = "#include \"";
-	size_t n, pos = 0;
+	 * include paths with spaces in them.
+	 */
 
-	while((n = source.find(include, pos)) != string::npos) {
-		size_t n_start = n + include.size();
-		size_t n_end = source.find("\"", n_start);
-		string filename = source.substr(n_start, n_end - n_start);
+	string result = "";
+	vector<string> lines;
+	string_split(lines, source, "\n", false);
 
-		string text, filepath = path_join(path, filename);
-
-		if(path_read_text(filepath, text)) {
-			text = path_source_replace_includes(text, path_dirname(filepath));
-			source.replace(n, n_end + 1 - n, "\n" + text + "\n");
+	for(size_t i = 0; i < lines.size(); ++i) {
+		string line = lines[i];
+		if(line[0] == '#') {
+			string token = string_strip(line.substr(1, line.size() - 1));
+			if(string_startswith(token, "include")) {
+				token = string_strip(token.substr(7, token.size() - 7));
+				if(token[0] == '"') {
+					size_t n_start = 1;
+					size_t n_end = token.find("\"", n_start);
+					string filename = token.substr(n_start, n_end - n_start);
+					string text, filepath = path_join(path, filename);
+					if(path_read_text(filepath, text)) {
+						/* Replace include directories with both current path
+						 * and path extracted from the include file.
+						 * Not totally robust, but works fine for Cycles kernel
+						 * and avoids having list of include directories.x
+						 */
+						text = path_source_replace_includes(
+						        text, path_dirname(filepath), filename);
+						text = path_source_replace_includes(text, path, filename);
+						/* Use line directives for better error messages. */
+						line = line_directive(filepath, 1)
+						     + token.replace(0, n_end + 1, "\n" + text + "\n")
+						     + line_directive(path_join(path, source_filename), i);
+					}
+				}
+			}
 		}
-		else
-			pos = n_end;
+		result += line + "\n";
 	}
 
-	return source;
+	return result;
 }
 
 FILE *path_fopen(const string& path, const string& mode)
 {
+#ifdef _WIN32
+	wstring path_wc = string_to_wstring(path);
+	wstring mode_wc = string_to_wstring(mode);
+	return _wfopen(path_wc.c_str(), mode_wc.c_str());
+#else
 	return fopen(path.c_str(), mode.c_str());
+#endif
 }
 
 void path_cache_clear_except(const string& name, const set<string>& except)
