@@ -23,7 +23,10 @@ CCL_NAMESPACE_BEGIN
 
 class Stats {
 public:
+	enum static_init_t { static_init = 0 };
+
 	Stats() : mem_used(0), mem_peak(0) {}
+	explicit Stats(static_init_t) {}
 
 	void mem_alloc(size_t size) {
 		atomic_add_z(&mem_used, size);
