@@ -16,8 +16,10 @@
 
 #include "util_debug.h"
 #include "util_foreach.h"
+#include "util_logging.h"
 #include "util_system.h"
 #include "util_task.h"
+#include "util_time.h"
 
 //#define THREADING_DEBUG_ENABLED
 
@@ -34,6 +36,7 @@ CCL_NAMESPACE_BEGIN
 
 TaskPool::TaskPool()
 {
+	num_tasks_handled = 0;
 	num = 0;
 	do_cancel = false;
 }
@@ -58,7 +61,7 @@ void TaskPool::push(const TaskRunFunction& run, bool front)
 	push(new Task(run), front);
 }
 
-void TaskPool::wait_work()
+void TaskPool::wait_work(Summary *stats)
 {
 	thread_scoped_lock num_lock(num_mutex);
 
@@ -89,7 +92,7 @@ void TaskPool::wait_work()
 		/* if found task, do it, otherwise wait until other tasks are done */
 		if(found_entry) {
 			/* run task */
-			work_entry.task->run();
+			work_entry.task->run(0);
 
 			/* delete task */
 			delete work_entry.task;
@@ -108,6 +111,11 @@ void TaskPool::wait_work()
 			THREADING_DEBUG("num==%d, condition wait done in TaskPool::wait_work !found_entry\n", num);
 		}
 	}
+
+	if(stats != NULL) {
+		stats->time_total = time_dt() - start_time;
+		stats->num_tasks_handled = num_tasks_handled;
+	}
 }
 
 void TaskPool::cancel()
@@ -158,7 +166,11 @@ void TaskPool::num_decrease(int done)
 void TaskPool::num_increase()
 {
 	thread_scoped_lock num_lock(num_mutex);
+	if(num_tasks_handled == 0) {
+		start_time = time_dt();
+	}
 	num++;
+	num_tasks_handled++;
 	THREADING_DEBUG("num==%d, notifying all in TaskPool::num_increase\n", num);
 	num_cond.notify_all();
 }
@@ -187,12 +199,30 @@ void TaskScheduler::init(int num_threads)
 			/* automatic number of threads */
 			num_threads = system_cpu_thread_count();
 		}
+		VLOG(1) << "Creating pool of " << num_threads << " threads.";
 
 		/* launch threads that will be waiting for work */
 		threads.resize(num_threads);
 
-		for(size_t i = 0; i < threads.size(); i++)
-			threads[i] = new thread(function_bind(&TaskScheduler::thread_run, i));
+		int num_groups = system_cpu_group_count();
+		int thread_index = 0;
+		for(int group = 0; group < num_groups; ++group) {
+			/* NOTE: That's not really efficient from threading point of view,
+			 * but it is simple to read and it doesn't make sense to use more
+			 * user-specified threads than logical threads anyway.
+			 */
+			int num_group_threads = (group == num_groups - 1)
+			        ? (threads.size() - thread_index)
+			        : system_cpu_group_thread_count(group);
+			for(int group_thread = 0;
+				group_thread < num_group_threads && thread_index < threads.size();
+				++group_thread, ++thread_index)
+			{
+				threads[thread_index] = new thread(function_bind(&TaskScheduler::thread_run,
+				                                                 thread_index + 1),
+				                                   group);
+			}
+		}
 	}
 	
 	users++;
@@ -206,8 +236,10 @@ void TaskScheduler::exit()
 
 	if(users == 0) {
 		/* stop all waiting threads */
+		TaskScheduler::queue_mutex.lock();
 		do_exit = true;
 		TaskScheduler::queue_cond.notify_all();
+		TaskScheduler::queue_mutex.unlock();
 
 		/* delete threads */
 		foreach(thread *t, threads) {
@@ -243,7 +275,7 @@ bool TaskScheduler::thread_wait_pop(Entry& entry)
 	return true;
 }
 
-void TaskScheduler::thread_run(int /*thread_id*/)
+void TaskScheduler::thread_run(int thread_id)
 {
 	Entry entry;
 
@@ -252,7 +284,7 @@ void TaskScheduler::thread_run(int /*thread_id*/)
 	/* keep popping off tasks */
 	while(thread_wait_pop(entry)) {
 		/* run task */
-		entry.task->run();
+		entry.task->run(thread_id);
 
 		/* delete task */
 		delete entry.task;
@@ -419,7 +451,7 @@ void DedicatedTaskPool::thread_run()
 	/* keep popping off tasks */
 	while(thread_wait_pop(task)) {
 		/* run task */
-		task->run();
+		task->run(0);
 
 		/* delete task */
 		delete task;
@@ -450,5 +482,13 @@ void DedicatedTaskPool::clear()
 	num_decrease(done);
 }
 
+string TaskPool::Summary::full_report() const
+{
+	string report = "";
+	report += string_printf("Total time:    %f\n", time_total);
+	report += string_printf("Tasks handled: %d\n", num_tasks_handled);
+	return report;
+}
+
 CCL_NAMESPACE_END
 
