@@ -21,8 +21,6 @@
  * ***** END GPL LICENSE BLOCK *****
  */
 
-#ifdef WITH_INPUT_NDOF
-
 #define DEBUG_NDOF_DRIVER false
 
 #include "GHOST_NDOFManagerCocoa.h"
@@ -115,13 +113,15 @@ static void* load_func(void* module, const char* func_name)
 
 #define LOAD_FUNC(name) name = (name##_ptr) load_func(module, #name)
 
+static void* module; // handle to the whole driver
+
 static bool load_driver_functions()
 {
 	if (driver_loaded) {
 		return true;
 	}
 
-	void* module = dlopen("3DconnexionClient.framework/3DconnexionClient", RTLD_LAZY | RTLD_LOCAL);
+	module = dlopen("3DconnexionClient.framework/3DconnexionClient", RTLD_LAZY | RTLD_LOCAL);
 
 	if (module) {
 		LOAD_FUNC(SetConnexionHandlers);
@@ -145,8 +145,6 @@ static bool load_driver_functions()
 
 			has_old_driver = (SetConnexionClientButtonMask == NULL);
 		}
-
-		dlclose(module); // functions will remain loaded
 	}
 #if DEBUG_NDOF_DRIVER
 	else {
@@ -161,6 +159,10 @@ static bool load_driver_functions()
 	return driver_loaded;
 	}
 
+static void unload_driver()
+{
+	dlclose(module);
+}
 
 static void DeviceAdded(uint32_t unused)
 {
@@ -198,8 +200,8 @@ static void DeviceEvent(uint32_t unused, uint32_t msg_type, void* msg_arg)
 				case kConnexionCmdHandleAxis:
 				{
 					// convert to blender view coordinates
-					const short t[3] = {s->axis[0], -(s->axis[2]), s->axis[1]};
-					const short r[3] = {-(s->axis[3]), s->axis[5], -(s->axis[4])};
+					const int t[3] = {s->axis[0], -(s->axis[2]), s->axis[1]};
+					const int r[3] = {-(s->axis[3]), s->axis[5], -(s->axis[4])};
 
 					ndof_manager->updateTranslation(t, now);
 					ndof_manager->updateRotation(r, now);
@@ -210,7 +212,9 @@ static void DeviceEvent(uint32_t unused, uint32_t msg_type, void* msg_arg)
 				case kConnexionCmdHandleButtons:
 				{
 					int button_bits = has_old_driver ? s->buttons8 : s->buttons;
+#ifdef DEBUG_NDOF_BUTTONS
 					printf("button bits: 0x%08x\n", button_bits);
+#endif
 					ndof_manager->updateButtons(button_bits, now);
 					ghost_system->notifyExternalEventProcessed();
 					break;
@@ -266,6 +270,7 @@ GHOST_NDOFManagerCocoa::~GHOST_NDOFManagerCocoa()
 	if (driver_loaded) {
 		UnregisterConnexionClient(clientID);
 		CleanupConnexionHandlers();
+		unload_driver();
 
 		ghost_system = NULL;
 		ndof_manager = NULL;
@@ -276,5 +281,3 @@ bool GHOST_NDOFManagerCocoa::available()
 {
 	return driver_loaded;
 }
-
-#endif // WITH_INPUT_NDOF
