@@ -59,6 +59,8 @@
 #include "BKE_global.h"
 #include "BKE_idprop.h"
 #include "BKE_library.h"
+#include "BKE_library_query.h"
+#include "BKE_library_remap.h"
 #include "BKE_main.h"
 #include "BKE_object.h"
 
@@ -92,94 +94,32 @@ bAction *add_empty_action(Main *bmain, const char name[])
 
 /* .................................. */
 
-/* temp data for BKE_action_make_local */
-typedef struct tMakeLocalActionContext {
-	bAction *act;       /* original action */
-	bAction *act_new;   /* new action */
-	
-	bool is_lib;        /* some action users were libraries */
-	bool is_local;      /* some action users were not libraries */
-} tMakeLocalActionContext;
-
-/* helper function for BKE_action_make_local() - local/lib init step */
-static void make_localact_init_cb(ID *id, AnimData *adt, void *mlac_ptr)
-{
-	tMakeLocalActionContext *mlac = (tMakeLocalActionContext *)mlac_ptr;
-	
-	if (adt->action == mlac->act) {
-		if (id->lib) mlac->is_lib = true;
-		else mlac->is_local = true;
-	}
-}
-
-/* helper function for BKE_action_make_local() - change references */
-static void make_localact_apply_cb(ID *id, AnimData *adt, void *mlac_ptr)
-{
-	tMakeLocalActionContext *mlac = (tMakeLocalActionContext *)mlac_ptr;
-	
-	if (adt->action == mlac->act) {
-		if (id->lib == NULL) {
-			adt->action = mlac->act_new;
-			
-			id_us_plus(&mlac->act_new->id);
-			id_us_min(&mlac->act->id);
-		}
-	}
-}
-
 // does copy_fcurve...
-void BKE_action_make_local(bAction *act)
+void BKE_action_make_local(Main *bmain, bAction *act, const bool lib_local)
 {
-	tMakeLocalActionContext mlac = {act, NULL, false, false};
-	Main *bmain = G.main;
-	
-	if (act->id.lib == NULL)
-		return;
-	
-	/* XXX: double-check this; it used to be just single-user check, but that was when fake-users were still default */
-	if ((act->id.flag & LIB_FAKEUSER) && (act->id.us <= 1)) {
-		id_clear_lib_data(bmain, &act->id);
-		return;
-	}
-	
-	BKE_animdata_main_cb(bmain, make_localact_init_cb, &mlac);
-	
-	if (mlac.is_local && mlac.is_lib == false) {
-		id_clear_lib_data(bmain, &act->id);
-	}
-	else if (mlac.is_local && mlac.is_lib) {
-		mlac.act_new = BKE_action_copy(act);
-		mlac.act_new->id.us = 0;
-
-		BKE_id_lib_local_paths(bmain, act->id.lib, &mlac.act_new->id);
-
-		BKE_animdata_main_cb(bmain, make_localact_apply_cb, &mlac);
-	}
+	BKE_id_make_local_generic(bmain, &act->id, true, lib_local);
 }
 
 /* .................................. */
 
+/** Free (or release) any data used by this action (does not free the action itself). */
 void BKE_action_free(bAction *act)
-{
-	/* sanity check */
-	if (act == NULL)
-		return;
-	
+{	
+	/* No animdata here. */
+
 	/* Free F-Curves */
 	free_fcurves(&act->curves);
 	
 	/* Free groups */
-	if (act->groups.first)
-		BLI_freelistN(&act->groups);
+	BLI_freelistN(&act->groups);
 		
 	/* Free pose-references (aka local markers) */
-	if (act->markers.first)
-		BLI_freelistN(&act->markers);
+	BLI_freelistN(&act->markers);
 }
 
 /* .................................. */
 
-bAction *BKE_action_copy(bAction *src)
+bAction *BKE_action_copy(Main *bmain, bAction *src)
 {
 	bAction *dst = NULL;
 	bActionGroup *dgrp, *sgrp;
@@ -187,7 +127,7 @@ bAction *BKE_action_copy(bAction *src)
 	
 	if (src == NULL) 
 		return NULL;
-	dst = BKE_libblock_copy(&src->id);
+	dst = BKE_libblock_copy(bmain, &src->id);
 	
 	/* duplicate the lists of groups and markers */
 	BLI_duplicatelist(&dst->groups, &src->groups);
@@ -216,9 +156,7 @@ bAction *BKE_action_copy(bAction *src)
 		}
 	}
 	
-	if (src->id.lib) {
-		BKE_id_lib_local_paths(G.main, src->id.lib, &dst->id);
-	}
+	BKE_id_copy_ensure_local(bmain, &src->id, &dst->id);
 
 	return dst;
 }
@@ -493,6 +431,8 @@ bPoseChannel *BKE_pose_channel_verify(bPose *pose, const char *name)
 	unit_axis_angle(chan->rotAxis, &chan->rotAngle);
 	chan->size[0] = chan->size[1] = chan->size[2] = 1.0f;
 	
+	chan->scaleIn = chan->scaleOut = 1.0f;
+	
 	chan->limitmin[0] = chan->limitmin[1] = chan->limitmin[2] = -180.0f;
 	chan->limitmax[0] = chan->limitmax[1] = chan->limitmax[2] = 180.0f;
 	chan->stiffness[0] = chan->stiffness[1] = chan->stiffness[2] = 0.0f;
@@ -596,7 +536,16 @@ void BKE_pose_copy_data(bPose **dst, bPose *src, const bool copy_constraints)
 	outPose = MEM_callocN(sizeof(bPose), "pose");
 	
 	BLI_duplicatelist(&outPose->chanbase, &src->chanbase);
-
+	
+	/* Rebuild ghash here too, so that name lookups below won't be too bad...
+	 * BUT this will have the penalty that the ghash will be built twice
+	 * if BKE_pose_rebuild() gets called after this...
+	 */
+	if (outPose->chanbase.first != outPose->chanbase.last) {
+		outPose->chanhash = NULL;
+		BKE_pose_channels_hash_make(outPose);
+	}
+	
 	outPose->iksolver = src->iksolver;
 	outPose->ikdata = NULL;
 	outPose->ikparam = MEM_dupallocN(src->ikparam);
@@ -608,10 +557,16 @@ void BKE_pose_copy_data(bPose **dst, bPose *src, const bool copy_constraints)
 			id_us_plus(&pchan->custom->id);
 		}
 
-		/* warning, O(n2) here, but it's a rarely used feature. */
+		/* warning, O(n2) here, if done without the hash, but these are rarely used features. */
 		if (pchan->custom_tx) {
 			pchan->custom_tx = BKE_pose_channel_find_name(outPose, pchan->custom_tx->name);
 		}
+		if (pchan->bbone_prev) {
+			pchan->bbone_prev = BKE_pose_channel_find_name(outPose, pchan->bbone_prev->name);
+		}
+		if (pchan->bbone_next) {
+			pchan->bbone_next = BKE_pose_channel_find_name(outPose, pchan->bbone_next->name);
+		}
 
 		if (copy_constraints) {
 			BKE_constraints_copy(&listb, &pchan->constraints, true);  // BKE_constraints_copy NULLs listb
@@ -726,7 +681,7 @@ void BKE_pose_channels_remove(
         Object *ob,
         bool (*filter_fn)(const char *bone_name, void *user_data), void *user_data)
 {
-	/*  First erase any associated pose channel */
+	/* Erase any associated pose channel, along with any references to them */
 	if (ob->pose) {
 		bPoseChannel *pchan, *pchan_next;
 		bConstraint *con;
@@ -735,6 +690,7 @@ void BKE_pose_channels_remove(
 			pchan_next = pchan->next;
 
 			if (filter_fn(pchan->name, user_data)) {
+				/* Bone itself is being removed */
 				BKE_pose_channel_free(pchan);
 				if (ob->pose->chanhash) {
 					BLI_ghash_remove(ob->pose->chanhash, pchan->name, NULL, NULL);
@@ -742,6 +698,7 @@ void BKE_pose_channels_remove(
 				BLI_freelinkN(&ob->pose->chanbase, pchan);
 			}
 			else {
+				/* Maybe something the bone references is being removed instead? */
 				for (con = pchan->constraints.first; con; con = con->next) {
 					const bConstraintTypeInfo *cti = BKE_constraint_typeinfo_get(con);
 					ListBase targets = {NULL, NULL};
@@ -765,6 +722,20 @@ void BKE_pose_channels_remove(
 							cti->flush_constraint_targets(con, &targets, 0);
 					}
 				}
+				
+				if (pchan->bbone_prev) {
+					if (filter_fn(pchan->bbone_prev->name, user_data))
+						pchan->bbone_prev = NULL;
+				}
+				if (pchan->bbone_next) {
+					if (filter_fn(pchan->bbone_next->name, user_data))
+						pchan->bbone_next = NULL;
+				}
+				
+				if (pchan->custom_tx) {
+					if (filter_fn(pchan->custom_tx->name, user_data))
+						pchan->custom_tx = NULL;
+				}
 			}
 		}
 	}
@@ -824,26 +795,35 @@ void BKE_pose_channels_free(bPose *pose)
 	BKE_pose_channels_free_ex(pose, true);
 }
 
+void BKE_pose_free_data_ex(bPose *pose, bool do_id_user)
+{
+	/* free pose-channels */
+	BKE_pose_channels_free_ex(pose, do_id_user);
+
+	/* free pose-groups */
+	if (pose->agroups.first)
+		BLI_freelistN(&pose->agroups);
+
+	/* free IK solver state */
+	BIK_clear_data(pose);
+
+	/* free IK solver param */
+	if (pose->ikparam)
+		MEM_freeN(pose->ikparam);
+}
+
+void BKE_pose_free_data(bPose *pose)
+{
+	BKE_pose_free_data_ex(pose, true);
+}
+
 /**
  * Removes and deallocates all data from a pose, and also frees the pose.
  */
 void BKE_pose_free_ex(bPose *pose, bool do_id_user)
 {
 	if (pose) {
-		/* free pose-channels */
-		BKE_pose_channels_free_ex(pose, do_id_user);
-		
-		/* free pose-groups */
-		if (pose->agroups.first)
-			BLI_freelistN(&pose->agroups);
-		
-		/* free IK solver state */
-		BIK_clear_data(pose);
-		
-		/* free IK solver param */
-		if (pose->ikparam)
-			MEM_freeN(pose->ikparam);
-		
+		BKE_pose_free_data_ex(pose, do_id_user);
 		/* free pose */
 		MEM_freeN(pose);
 	}
@@ -869,6 +849,15 @@ static void copy_pose_channel_data(bPoseChannel *pchan, const bPoseChannel *chan
 	copy_m4_m4(pchan->pose_mat, (float(*)[4])chan->pose_mat);
 	pchan->flag = chan->flag;
 	
+	pchan->roll1 = chan->roll1;
+	pchan->roll2 = chan->roll2;
+	pchan->curveInX = chan->curveInX;
+	pchan->curveInY = chan->curveInY;
+	pchan->curveOutX = chan->curveOutX;
+	pchan->curveOutY = chan->curveOutY;
+	pchan->scaleIn = chan->scaleIn;
+	pchan->scaleOut = chan->scaleOut;
+	
 	con = chan->constraints.first;
 	for (pcon = pchan->constraints.first; pcon && con; pcon = pcon->next, con = con->next) {
 		pcon->enforce = con->enforce;
@@ -879,7 +868,7 @@ static void copy_pose_channel_data(bPoseChannel *pchan, const bPoseChannel *chan
 /**
  * Copy the internal members of each pose channel including constraints
  * and ID-Props, used when duplicating bones in editmode.
- * (unlike copy_pose_channel_data which only).
+ * (unlike copy_pose_channel_data which only does posing-related stuff).
  *
  * \note use when copying bones in editmode (on returned value from #BKE_pose_channel_verify)
  */
@@ -902,6 +891,11 @@ void BKE_pose_channel_copy_data(bPoseChannel *pchan, const bPoseChannel *pchan_f
 	pchan->ikstretch = pchan_from->ikstretch;
 	pchan->ikrotweight = pchan_from->ikrotweight;
 	pchan->iklinweight = pchan_from->iklinweight;
+	
+	/* bbone settings (typically not animated) */
+	pchan->bboneflag = pchan_from->bboneflag;
+	pchan->bbone_next = pchan_from->bbone_next;
+	pchan->bbone_prev = pchan_from->bbone_prev;
 
 	/* constraints */
 	BKE_constraints_copy(&pchan->constraints, &pchan_from->constraints, true);
@@ -922,6 +916,8 @@ void BKE_pose_channel_copy_data(bPoseChannel *pchan, const bPoseChannel *pchan_f
 	if (pchan->custom) {
 		id_us_plus(&pchan->custom->id);
 	}
+
+	pchan->custom_scale = pchan_from->custom_scale;
 }
 
 
@@ -1269,6 +1265,18 @@ short action_get_item_transforms(bAction *act, Object *ob, bPoseChannel *pchan,
 				}
 			}
 			
+			if ((curves) || (flags & ACT_TRANS_BBONE) == 0) {
+				/* bbone shape properties */
+				pPtr = strstr(bPtr, "bbone_");
+				if (pPtr) {
+					flags |= ACT_TRANS_BBONE;
+					
+					if (curves)
+						BLI_addtail(curves, BLI_genericNodeN(fcu));
+					continue;
+				}
+			}
+			
 			if ((curves) || (flags & ACT_TRANS_PROP) == 0) {
 				/* custom properties only */
 				pPtr = strstr(bPtr, "[\""); /* extra '"' comment here to keep my texteditor functionlist working :) */
@@ -1328,8 +1336,13 @@ void BKE_pose_rest(bPose *pose)
 		unit_qt(pchan->quat);
 		unit_axis_angle(pchan->rotAxis, &pchan->rotAngle);
 		pchan->size[0] = pchan->size[1] = pchan->size[2] = 1.0f;
-
-		pchan->flag &= ~(POSE_LOC | POSE_ROT | POSE_SIZE);
+		
+		pchan->roll1 = pchan->roll2 = 0.0f;
+		pchan->curveInX = pchan->curveInY = 0.0f;
+		pchan->curveOutX = pchan->curveOutY = 0.0f;
+		pchan->scaleIn = pchan->scaleOut = 1.0f;
+		
+		pchan->flag &= ~(POSE_LOC | POSE_ROT | POSE_SIZE | POSE_BBONE_SHAPE);
 	}
 }
 
@@ -1364,9 +1377,19 @@ bool BKE_pose_copy_result(bPose *to, bPose *from)
 			copy_v3_v3(pchanto->pose_head, pchanfrom->pose_head);
 			copy_v3_v3(pchanto->pose_tail, pchanfrom->pose_tail);
 			
+			pchanto->roll1 = pchanfrom->roll1;
+			pchanto->roll2 = pchanfrom->roll2;
+			pchanto->curveInX = pchanfrom->curveInX;
+			pchanto->curveInY = pchanfrom->curveInY;
+			pchanto->curveOutX = pchanfrom->curveOutX;
+			pchanto->curveOutY = pchanfrom->curveOutY;
+			pchanto->scaleIn = pchanfrom->scaleIn;
+			pchanto->scaleOut = pchanfrom->scaleOut;
+			
 			pchanto->rotmode = pchanfrom->rotmode;
 			pchanto->flag = pchanfrom->flag;
 			pchanto->protectflag = pchanfrom->protectflag;
+			pchanto->bboneflag = pchanfrom->bboneflag;
 		}
 	}
 	return true;
@@ -1412,6 +1435,18 @@ void what_does_obaction(Object *ob, Object *workob, bPose *pose, bAction *act, c
 	workob->constraints.last = ob->constraints.last;
 	
 	workob->pose = pose; /* need to set pose too, since this is used for both types of Action Constraint */
+	if (pose) {
+		/* This function is most likely to be used with a temporary pose with a single bone in there.
+		 * For such cases it makes no sense to create hash since it'll only waste CPU ticks on memory
+		 * allocation and also will make lookup slower.
+		 */
+		if (pose->chanbase.first != pose->chanbase.last) {
+			BKE_pose_channels_hash_make(pose);
+		}
+		if (pose->flag & POSE_CONSTRAINTS_NEED_UPDATE_FLAGS) {
+			BKE_pose_update_constraint_flags(pose);
+		}
+	}
 
 	BLI_strncpy(workob->parsubstr, ob->parsubstr, sizeof(workob->parsubstr));
 	BLI_strncpy(workob->id.name, "OB<ConstrWorkOb>", sizeof(workob->id.name)); /* we don't use real object name, otherwise RNA screws with the real thing */
