@@ -47,31 +47,6 @@
 
 #include "intern/bmesh_private.h"
 
-/**
- * \brief TEST EDGE SIDE and POINT IN TRIANGLE
- *
- * Point in triangle tests stolen from scanfill.c.
- * Used for tessellator
- */
-
-static bool testedgesidef(const float v1[2], const float v2[2], const float v3[2])
-{
-	/* is v3 to the right of v1 - v2 ? With exception: v3 == v1 || v3 == v2 */
-	double inp;
-
-	//inp = (v2[cox] - v1[cox]) * (v1[coy] - v3[coy]) + (v1[coy] - v2[coy]) * (v1[cox] - v3[cox]);
-	inp = (v2[0] - v1[0]) * (v1[1] - v3[1]) + (v1[1] - v2[1]) * (v1[0] - v3[0]);
-
-	if (inp < 0.0) {
-		return false;
-	}
-	else if (inp == 0) {
-		if (v1[0] == v3[0] && v1[1] == v3[1]) return false;
-		if (v2[0] == v3[0] && v2[1] == v3[1]) return false;
-	}
-	return true;
-}
-
 /**
  * \brief COMPUTE POLY NORMAL (BMFace)
  *
@@ -286,64 +261,258 @@ float BM_face_calc_perimeter(const BMFace *f)
 	return perimeter;
 }
 
-void BM_vert_tri_calc_plane(BMVert *verts[3], float r_plane[3])
+/**
+ * Utility function to calculate the edge which is most different from the other two.
+ *
+ * \return The first edge index, where the second vertex is ``(index + 1) % 3``.
+ */
+static int bm_vert_tri_find_unique_edge(BMVert *verts[3])
 {
-	float lens[3];
+	/* find the most 'unique' loop, (greatest difference to others) */
+#if 1
+	/* optimized version that avoids sqrt */
 	float difs[3];
-	int  order[3] = {0, 1, 2};
+	for (int i_prev = 1, i_curr = 2, i_next = 0;
+	     i_next < 3;
+	     i_prev = i_curr, i_curr = i_next++)
+	{
+		const float *co = verts[i_curr]->co;
+		const float *co_other[2] = {verts[i_prev]->co, verts[i_next]->co};
+		float proj_dir[3];
+		mid_v3_v3v3(proj_dir, co_other[0], co_other[1]);
+		sub_v3_v3(proj_dir, co);
 
-	lens[0] = len_v3v3(verts[0]->co, verts[1]->co);
-	lens[1] = len_v3v3(verts[1]->co, verts[2]->co);
-	lens[2] = len_v3v3(verts[2]->co, verts[0]->co);
-
-	/* find the shortest or the longest loop */
-	difs[0] = fabsf(lens[1] - lens[2]);
-	difs[1] = fabsf(lens[2] - lens[0]);
-	difs[2] = fabsf(lens[0] - lens[1]);
+		float proj_pair[2][3];
+		project_v3_v3v3(proj_pair[0], co_other[0], proj_dir);
+		project_v3_v3v3(proj_pair[1], co_other[1], proj_dir);
+		difs[i_next] = len_squared_v3v3(proj_pair[0], proj_pair[1]);
+	}
+#else
+	const float lens[3] = {
+		len_v3v3(verts[0]->co, verts[1]->co),
+		len_v3v3(verts[1]->co, verts[2]->co),
+		len_v3v3(verts[2]->co, verts[0]->co),
+	};
+	const float difs[3] = {
+		fabsf(lens[1] - lens[2]),
+		fabsf(lens[2] - lens[0]),
+		fabsf(lens[0] - lens[1]),
+	};
+#endif
 
+	int order[3] = {0, 1, 2};
 	axis_sort_v3(difs, order);
-	sub_v3_v3v3(r_plane, verts[order[0]]->co, verts[(order[0] + 1) % 3]->co);
+
+	return order[0];
 }
 
 /**
- * Compute a meaningful direction along the face (use for manipulator axis).
- * \note result isnt normalized.
+ * Calculate a tangent from any 3 vertices.
+ *
+ * The tangent aligns to the most *unique* edge
+ * (the edge most unlike the other two).
+ *
+ * \param r_tangent: Calculated unit length tangent (return value).
  */
-void BM_face_calc_plane(const BMFace *f, float r_plane[3])
+void BM_vert_tri_calc_tangent_edge(BMVert *verts[3], float r_tangent[3])
+{
+	const int index = bm_vert_tri_find_unique_edge(verts);
+
+	sub_v3_v3v3(r_tangent, verts[index]->co, verts[(index + 1) % 3]->co);
+
+	normalize_v3(r_tangent);
+}
+
+/**
+ * Calculate a tangent from any 3 vertices,
+ *
+ * The tangent follows the center-line formed by the most unique edges center
+ * and the opposite vertex.
+ *
+ * \param r_tangent: Calculated unit length tangent (return value).
+ */
+void BM_vert_tri_calc_tangent_edge_pair(BMVert *verts[3], float r_tangent[3])
+{
+	const int index = bm_vert_tri_find_unique_edge(verts);
+
+	const float *v_a     = verts[index]->co;
+	const float *v_b     = verts[(index + 1) % 3]->co;
+	const float *v_other = verts[(index + 2) % 3]->co;
+
+	mid_v3_v3v3(r_tangent, v_a, v_b);
+	sub_v3_v3v3(r_tangent, v_other, r_tangent);
+
+	normalize_v3(r_tangent);
+}
+
+/**
+ * Compute the tangent of the face, using the longest edge.
+ */
+void  BM_face_calc_tangent_edge(const BMFace *f, float r_tangent[3])
+{
+	const BMLoop *l_long  = BM_face_find_longest_loop((BMFace *)f);
+
+	sub_v3_v3v3(r_tangent, l_long->v->co, l_long->next->v->co);
+
+	normalize_v3(r_tangent);
+
+}
+
+/**
+ * Compute the tangent of the face, using the two longest disconnected edges.
+ *
+ * \param r_tangent: Calculated unit length tangent (return value).
+ */
+void  BM_face_calc_tangent_edge_pair(const BMFace *f, float r_tangent[3])
 {
 	if (f->len == 3) {
 		BMVert *verts[3];
 
 		BM_face_as_array_vert_tri((BMFace *)f, verts);
 
-		BM_vert_tri_calc_plane(verts, r_plane);
+		BM_vert_tri_calc_tangent_edge_pair(verts, r_tangent);
 	}
 	else if (f->len == 4) {
+		/* Use longest edge pair */
 		BMVert *verts[4];
 		float vec[3], vec_a[3], vec_b[3];
 
-		// BM_iter_as_array(NULL, BM_VERTS_OF_FACE, efa, (void **)verts, 4);
 		BM_face_as_array_vert_quad((BMFace *)f, verts);
 
 		sub_v3_v3v3(vec_a, verts[3]->co, verts[2]->co);
 		sub_v3_v3v3(vec_b, verts[0]->co, verts[1]->co);
-		add_v3_v3v3(r_plane, vec_a, vec_b);
+		add_v3_v3v3(r_tangent, vec_a, vec_b);
 
 		sub_v3_v3v3(vec_a, verts[0]->co, verts[3]->co);
 		sub_v3_v3v3(vec_b, verts[1]->co, verts[2]->co);
 		add_v3_v3v3(vec, vec_a, vec_b);
-		/* use the biggest edge length */
-		if (len_squared_v3(r_plane) < len_squared_v3(vec)) {
-			copy_v3_v3(r_plane, vec);
+		/* use the longest edge length */
+		if (len_squared_v3(r_tangent) < len_squared_v3(vec)) {
+			copy_v3_v3(r_tangent, vec);
 		}
 	}
 	else {
-		const BMLoop *l_long  = BM_face_find_longest_loop((BMFace *)f);
+		/* For ngons use two longest disconnected edges */
+		BMLoop *l_long = BM_face_find_longest_loop((BMFace *)f);
+		BMLoop *l_long_other = NULL;
 
-		sub_v3_v3v3(r_plane, l_long->v->co, l_long->next->v->co);
+		float len_max_sq = 0.0f;
+		float vec_a[3], vec_b[3];
+
+		BMLoop *l_iter = l_long->prev->prev;
+		BMLoop *l_last = l_long->next;
+
+		do {
+			const float len_sq = len_squared_v3v3(l_iter->v->co, l_iter->next->v->co);
+			if (len_sq >= len_max_sq) {
+				l_long_other = l_iter;
+				len_max_sq = len_sq;
+			}
+		} while ((l_iter = l_iter->prev) != l_last);
+
+		sub_v3_v3v3(vec_a, l_long->next->v->co, l_long->v->co);
+		sub_v3_v3v3(vec_b, l_long_other->v->co, l_long_other->next->v->co);
+		add_v3_v3v3(r_tangent, vec_a, vec_b);
+
+		/* Edges may not be opposite side of the ngon,
+		 * this could cause problems for ngons with multiple-aligned edges of the same length.
+		 * Fallback to longest edge. */
+		if (UNLIKELY(normalize_v3(r_tangent) == 0.0f)) {
+			normalize_v3_v3(r_tangent, vec_a);
+		}
 	}
+}
 
-	normalize_v3(r_plane);
+/**
+ * Compute the tangent of the face, using the edge farthest away from any vertex in the face.
+ *
+ * \param r_tangent: Calculated unit length tangent (return value).
+ */
+void  BM_face_calc_tangent_edge_diagonal(const BMFace *f, float r_tangent[3])
+{
+	BMLoop *l_iter, *l_first;
+
+	l_iter = l_first = BM_FACE_FIRST_LOOP(f);
+
+	/* incase of degenerate faces */
+	zero_v3(r_tangent);
+
+	/* warning: O(n^2) loop here, take care! */
+	float dist_max_sq = 0.0f;
+	do {
+		BMLoop *l_iter_other = l_iter->next;
+		BMLoop *l_iter_last = l_iter->prev;
+		do {
+			BLI_assert(!ELEM(l_iter->v->co, l_iter_other->v->co, l_iter_other->next->v->co));
+			float co_other[3], vec[3];
+			closest_to_line_segment_v3(co_other, l_iter->v->co, l_iter_other->v->co, l_iter_other->next->v->co);
+			sub_v3_v3v3(vec, l_iter->v->co, co_other);
+
+			const float dist_sq = len_squared_v3(vec);
+			if (dist_sq > dist_max_sq) {
+				dist_max_sq = dist_sq;
+				copy_v3_v3(r_tangent, vec);
+			}
+		} while ((l_iter_other = l_iter_other->next) != l_iter_last);
+	} while ((l_iter = l_iter->next) != l_first);
+
+	normalize_v3(r_tangent);
+}
+
+/**
+ * Compute the tangent of the face, using longest distance between vertices on the face.
+ *
+ * \note The logic is almost identical to #BM_face_calc_tangent_edge_diagonal
+ */
+void  BM_face_calc_tangent_vert_diagonal(const BMFace *f, float r_tangent[3])
+{
+	BMLoop *l_iter, *l_first;
+
+	l_iter = l_first = BM_FACE_FIRST_LOOP(f);
+
+	/* incase of degenerate faces */
+	zero_v3(r_tangent);
+
+	/* warning: O(n^2) loop here, take care! */
+	float dist_max_sq = 0.0f;
+	do {
+		BMLoop *l_iter_other = l_iter->next;
+		do {
+			float vec[3];
+			sub_v3_v3v3(vec, l_iter->v->co, l_iter_other->v->co);
+
+			const float dist_sq = len_squared_v3(vec);
+			if (dist_sq > dist_max_sq) {
+				dist_max_sq = dist_sq;
+				copy_v3_v3(r_tangent, vec);
+			}
+		} while ((l_iter_other = l_iter_other->next) != l_iter);
+	} while ((l_iter = l_iter->next) != l_first);
+
+	normalize_v3(r_tangent);
+}
+
+/**
+ * Compute a meaningful direction along the face (use for manipulator axis).
+ *
+ * \note Callers shouldn't depend on the *exact* method used here.
+ */
+void BM_face_calc_tangent_auto(const BMFace *f, float r_tangent[3])
+{
+	if (f->len == 3) {
+		/* most 'unique' edge of a triangle */
+		BMVert *verts[3];
+		BM_face_as_array_vert_tri((BMFace *)f, verts);
+		BM_vert_tri_calc_tangent_edge(verts, r_tangent);
+	}
+	else if (f->len == 4) {
+		/* longest edge pair of a quad */
+		BM_face_calc_tangent_edge_pair((BMFace *)f, r_tangent);
+	}
+	else {
+		/* longest edge of an ngon */
+		BM_face_calc_tangent_edge((BMFace *)f, r_tangent);
+	}
 }
 
 /**
@@ -408,29 +577,6 @@ void BM_face_calc_center_mean_weighted(const BMFace *f, float r_cent[3])
 		mul_v3_fl(r_cent, 1.0f / (float) totw);
 }
 
-/**
- * \brief BM LEGAL EDGES
- *
- * takes in a face and a list of edges, and sets to NULL any edge in
- * the list that bridges a concave region of the face or intersects
- * any of the faces's edges.
- */
-static void scale_edge_v2f(float v1[2], float v2[2], const float fac)
-{
-	float mid[2];
-
-	mid_v2_v2v2(mid, v1, v2);
-
-	sub_v2_v2v2(v1, v1, mid);
-	sub_v2_v2v2(v2, v2, mid);
-
-	mul_v2_fl(v1, fac);
-	mul_v2_fl(v2, fac);
-
-	add_v2_v2v2(v1, v1, mid);
-	add_v2_v2v2(v2, v2, mid);
-}
-
 /**
  * \brief POLY ROTATE PLANE
  *
@@ -716,67 +862,6 @@ void BM_face_normal_flip(BMesh *bm, BMFace *f)
 	BM_face_normal_flip_ex(bm, f, cd_loop_mdisp_offset, true);
 }
 
-/* detects if two line segments cross each other (intersects).
- * note, there could be more winding cases then there needs to be. */
-static bool line_crosses_v2f(const float v1[2], const float v2[2], const float v3[2], const float v4[2])
-{
-
-#define GETMIN2_AXIS(a, b, ma, mb, axis)   \
-	{                                      \
-		ma[axis] = min_ff(a[axis], b[axis]); \
-		mb[axis] = max_ff(a[axis], b[axis]); \
-	} (void)0
-
-#define GETMIN2(a, b, ma, mb)          \
-	{                                  \
-		GETMIN2_AXIS(a, b, ma, mb, 0); \
-		GETMIN2_AXIS(a, b, ma, mb, 1); \
-	} (void)0
-
-#define EPS (FLT_EPSILON * 15)
-
-	int w1, w2, w3, w4, w5 /*, re */;
-	float mv1[2], mv2[2], mv3[2], mv4[2];
-	
-	/* now test winding */
-	w1 = testedgesidef(v1, v3, v2);
-	w2 = testedgesidef(v2, v4, v1);
-	w3 = !testedgesidef(v1, v2, v3);
-	w4 = testedgesidef(v3, v2, v4);
-	w5 = !testedgesidef(v3, v1, v4);
-	
-	if (w1 == w2 && w2 == w3 && w3 == w4 && w4 == w5) {
-		return true;
-	}
-	
-	GETMIN2(v1, v2, mv1, mv2);
-	GETMIN2(v3, v4, mv3, mv4);
-	
-	/* do an interval test on the x and y axes */
-	/* first do x axis */
-	if (fabsf(v1[1] - v2[1]) < EPS &&
-	    fabsf(v3[1] - v4[1]) < EPS &&
-	    fabsf(v1[1] - v3[1]) < EPS)
-	{
-		return (mv4[0] >= mv1[0] && mv3[0] <= mv2[0]);
-	}
-
-	/* now do y axis */
-	if (fabsf(v1[0] - v2[0]) < EPS &&
-	    fabsf(v3[0] - v4[0]) < EPS &&
-	    fabsf(v1[0] - v3[0]) < EPS)
-	{
-		return (mv4[1] >= mv1[1] && mv3[1] <= mv2[1]);
-	}
-
-	return false;
-
-#undef GETMIN2_AXIS
-#undef GETMIN2
-#undef EPS
-
-}
-
 /**
  *  BM POINT IN FACE
  *
@@ -842,6 +927,7 @@ void BM_face_triangulate(
         const int quad_method,
         const int ngon_method,
         const bool use_tag,
+        /* use for ngons only! */
         MemArena *pf_arena,
 
         /* use for MOD_TRIANGULATE_NGON_BEAUTY only! */
@@ -1073,121 +1159,103 @@ void BM_face_triangulate(
  */
 void BM_face_splits_check_legal(BMesh *bm, BMFace *f, BMLoop *(*loops)[2], int len)
 {
-	const int len2 = len * 2;
-	BMLoop *l;
-	float v1[2], v2[2], v3[2], mid[2], *p1, *p2, *p3, *p4;
 	float out[2] = {-FLT_MAX, -FLT_MAX};
+	float center[2] = {0.0f, 0.0f};
 	float axis_mat[3][3];
 	float (*projverts)[2] = BLI_array_alloca(projverts, f->len);
-	float (*edgeverts)[2] = BLI_array_alloca(edgeverts, len2);
-	float fac1 = 1.0000001f, fac2 = 0.9f; //9999f; //0.999f;
-	int i, j, a = 0, clen;
+	const float *(*edgeverts)[2] = BLI_array_alloca(edgeverts, len);
+	BMLoop *l;
+	int i, i_prev, j;
 
 	BLI_assert(BM_face_is_normal_valid(f));
 
 	axis_dominant_v3_to_m3(axis_mat, f->no);
 
 	for (i = 0, l = BM_FACE_FIRST_LOOP(f); i < f->len; i++, l = l->next) {
-		BM_elem_index_set(l, i);  /* set_dirty */
 		mul_v2_m3v3(projverts[i], axis_mat, l->v->co);
+		add_v2_v2(center, projverts[i]);
 	}
-	bm->elem_index_dirty |= BM_LOOP;
 
 	/* first test for completely convex face */
 	if (is_poly_convex_v2((const float (*)[2])projverts, f->len)) {
 		return;
 	}
 
+	mul_v2_fl(center, 1.0f / f->len);
+
 	for (i = 0, l = BM_FACE_FIRST_LOOP(f); i < f->len; i++, l = l->next) {
+		BM_elem_index_set(l, i);  /* set_dirty */
+
+		/* center the projection for maximum accuracy */
+		sub_v2_v2(projverts[i], center);
+
 		out[0] = max_ff(out[0], projverts[i][0]);
 		out[1] = max_ff(out[1], projverts[i][1]);
 	}
+	bm->elem_index_dirty |= BM_LOOP;
 	
 	/* ensure we are well outside the face bounds (value is arbitrary) */
 	add_v2_fl(out, 1.0f);
 
 	for (i = 0; i < len; i++) {
-		copy_v2_v2(edgeverts[a + 0], projverts[BM_elem_index_get(loops[i][0])]);
-		copy_v2_v2(edgeverts[a + 1], projverts[BM_elem_index_get(loops[i][1])]);
-		scale_edge_v2f(edgeverts[a + 0], edgeverts[a + 1], fac2);
-		a += 2;
+		edgeverts[i][0] = projverts[BM_elem_index_get(loops[i][0])];
+		edgeverts[i][1] = projverts[BM_elem_index_get(loops[i][1])];
 	}
 
 	/* do convexity test */
 	for (i = 0; i < len; i++) {
-		copy_v2_v2(v2, edgeverts[i * 2 + 0]);
-		copy_v2_v2(v3, edgeverts[i * 2 + 1]);
-
-		mid_v2_v2v2(mid, v2, v3);
+		float mid[2];
+		mid_v2_v2v2(mid, edgeverts[i][0], edgeverts[i][1]);
 		
-		clen = 0;
-		for (j = 0; j < f->len; j++) {
-			p1 = projverts[j];
-			p2 = projverts[(j + 1) % f->len];
-			
-#if 0
-			copy_v2_v2(v1, p1);
-			copy_v2_v2(v2, p2);
-
-			scale_edge_v2f(v1, v2, fac1);
-			if (line_crosses_v2f(v1, v2, mid, out)) {
-				clen++;
+		int isect = 0;
+		int j_prev;
+		for (j = 0, j_prev = f->len - 1; j < f->len; j_prev = j++) {
+			const float *f_edge[2] = {projverts[j_prev], projverts[j]};
+			if (isect_seg_seg_v2(UNPACK2(f_edge), mid, out) == ISECT_LINE_LINE_CROSS) {
+				isect++;
 			}
-#else
-			if (line_crosses_v2f(p1, p2, mid, out)) {
-				clen++;
-			}
-#endif
 		}
 
-		if (clen % 2 == 0) {
+		if (isect % 2 == 0) {
 			loops[i][0] = NULL;
 		}
 	}
 
+#define EDGE_SHARE_VERT(e1, e2) \
+	((ELEM((e1)[0], (e2)[0], (e2)[1])) || \
+	 (ELEM((e1)[1], (e2)[0], (e2)[1])))
+
 	/* do line crossing tests */
-	for (i = 0; i < f->len; i++) {
-		p1 = projverts[i];
-		p2 = projverts[(i + 1) % f->len];
-		
-		copy_v2_v2(v1, p1);
-		copy_v2_v2(v2, p2);
-
-		scale_edge_v2f(v1, v2, fac1);
-
+	for (i = 0, i_prev = f->len - 1; i < f->len; i_prev = i++) {
+		const float *f_edge[2] = {projverts[i_prev], projverts[i]};
 		for (j = 0; j < len; j++) {
-			if (!loops[j][0]) {
-				continue;
-			}
-
-			p3 = edgeverts[j * 2];
-			p4 = edgeverts[j * 2 + 1];
-
-			if (line_crosses_v2f(v1, v2, p3, p4)) {
-				loops[j][0] = NULL;
-			}
-		}
-	}
-
-	for (i = 0; i < len; i++) {
-		for (j = 0; j < len; j++) {
-			if (j != i && loops[i][0] && loops[j][0]) {
-				p1 = edgeverts[i * 2];
-				p2 = edgeverts[i * 2 + 1];
-				p3 = edgeverts[j * 2];
-				p4 = edgeverts[j * 2 + 1];
-
-				copy_v2_v2(v1, p1);
-				copy_v2_v2(v2, p2);
-
-				scale_edge_v2f(v1, v2, fac1);
-
-				if (line_crosses_v2f(v1, v2, p3, p4)) {
-					loops[i][0] = NULL;
+			if ((loops[j][0] != NULL) &&
+			    !EDGE_SHARE_VERT(f_edge, edgeverts[j]))
+			{
+				if (isect_seg_seg_v2(UNPACK2(f_edge), UNPACK2(edgeverts[j])) == ISECT_LINE_LINE_CROSS) {
+					loops[j][0] = NULL;
 				}
 			}
 		}
 	}
+
+	/* self intersect tests */
+	for (i = 0; i < len; i++) {
+		if (loops[i][0]) {
+			for (j = i + 1; j < len; j++) {
+				if ((loops[j][0] != NULL) &&
+				    !EDGE_SHARE_VERT(edgeverts[i], edgeverts[j]))
+				{
+					if (isect_seg_seg_v2(UNPACK2(edgeverts[i]), UNPACK2(edgeverts[j])) == ISECT_LINE_LINE_CROSS) {
+						loops[i][0] = NULL;
+						break;
+					}
+				}
+			}
+		}
+	}
+
+#undef EDGE_SHARE_VERT
 }
 
 /**
