@@ -502,7 +502,7 @@ void ED_region_do_draw(bContext *C, ARegion *ar)
 	/* note; this sets state, so we can use wmOrtho and friends */
 	wmSubWindowScissorSet(win, ar->swinid, &ar->drawrct, scissor_pad);
 
-	wmOrtho2_region_ui(ar);
+	wmOrtho2_region_pixelspace(ar);
 	
 	UI_SetTheme(sa ? sa->spacetype : 0, at->regionid);
 	
@@ -580,15 +580,20 @@ void ED_region_tag_refresh_ui(ARegion *ar)
 void ED_region_tag_redraw_partial(ARegion *ar, const rcti *rct)
 {
 	if (ar && !(ar->do_draw & RGN_DRAWING)) {
-		if (!(ar->do_draw & RGN_DRAW)) {
+		if (!(ar->do_draw & (RGN_DRAW | RGN_DRAW_PARTIAL))) {
 			/* no redraw set yet, set partial region */
 			ar->do_draw |= RGN_DRAW_PARTIAL;
 			ar->drawrct = *rct;
 		}
 		else if (ar->drawrct.xmin != ar->drawrct.xmax) {
+			BLI_assert((ar->do_draw & RGN_DRAW_PARTIAL) != 0);
 			/* partial redraw already set, expand region */
 			BLI_rcti_union(&ar->drawrct, rct);
 		}
+		else {
+			BLI_assert((ar->do_draw & RGN_DRAW) != 0);
+			/* Else, full redraw is already requested, nothing to do here. */
+		}
 	}
 }
 
@@ -635,8 +640,8 @@ void ED_area_headerprint(ScrArea *sa, const char *str)
 		if (ar->regiontype == RGN_TYPE_HEADER) {
 			if (str) {
 				if (ar->headerstr == NULL)
-					ar->headerstr = MEM_mallocN(256, "headerprint");
-				BLI_strncpy(ar->headerstr, str, 256);
+					ar->headerstr = MEM_mallocN(UI_MAX_DRAW_STR, "headerprint");
+				BLI_strncpy(ar->headerstr, str, UI_MAX_DRAW_STR);
 			}
 			else if (ar->headerstr) {
 				MEM_freeN(ar->headerstr);
@@ -1507,6 +1512,16 @@ void ED_region_init(bContext *C, ARegion *ar)
 	region_update_rect(ar);
 }
 
+void ED_region_cursor_set(wmWindow *win, ScrArea *sa, ARegion *ar)
+{
+	if (ar && sa && ar->type && ar->type->cursor) {
+		ar->type->cursor(win, sa, ar);
+	}
+	else {
+		WM_cursor_set(win, CURSOR_STD);
+	}
+}
+
 /* for quick toggle, can skip fades */
 void region_toggle_hidden(bContext *C, ARegion *ar, const bool do_fade)
 {
@@ -1611,15 +1626,31 @@ void ED_area_swapspace(bContext *C, ScrArea *sa1, ScrArea *sa2)
 	ED_area_tag_refresh(sa2);
 }
 
-void ED_area_newspace(bContext *C, ScrArea *sa, int type)
+/**
+ * \param skip_ar_exit  Skip calling area exit callback. Set for opening temp spaces.
+ */
+void ED_area_newspace(bContext *C, ScrArea *sa, int type, const bool skip_ar_exit)
 {
 	if (sa->spacetype != type) {
 		SpaceType *st;
 		SpaceLink *slold;
 		SpaceLink *sl;
+		/* store sa->type->exit callback */
+		void *sa_exit = sa->type ? sa->type->exit : NULL;
+
+		/* in some cases (opening temp space) we don't want to
+		 * call area exit callback, so we temporarily unset it */
+		if (skip_ar_exit && sa->type) {
+			sa->type->exit = NULL;
+		}
 
 		ED_area_exit(C, sa);
 
+		/* restore old area exit callback */
+		if (skip_ar_exit && sa->type) {
+			sa->type->exit = sa_exit;
+		}
+
 		st = BKE_spacetype_from_id(type);
 		slold = sa->spacedata.first;
 
@@ -1686,12 +1717,12 @@ void ED_area_prevspace(bContext *C, ScrArea *sa)
 	SpaceLink *sl = sa->spacedata.first;
 
 	if (sl && sl->next) {
-		/* workaround for case of double prevspace, render window
-		 * with a file browser on top of it */
-		if (sl->next->spacetype == SPACE_FILE && sl->next->next)
-			ED_area_newspace(C, sa, sl->next->next->spacetype);
-		else
-			ED_area_newspace(C, sa, sl->next->spacetype);
+		ED_area_newspace(C, sa, sl->next->spacetype, false);
+
+		/* keep old spacedata but move it to end, so calling
+		 * ED_area_prevspace once more won't open it again */
+		BLI_remlink(&sa->spacedata, sl);
+		BLI_addtail(&sa->spacedata, sl);
 	}
 	else {
 		/* no change */
@@ -1737,7 +1768,7 @@ void ED_region_panels(const bContext *C, ARegion *ar, const char *context, int c
 	int redo;
 	int scroll;
 
-	bool use_category_tabs = (ar->regiontype == RGN_TYPE_TOOLS);  /* XXX, should use some better check? */
+	bool use_category_tabs = (ELEM(ar->regiontype, RGN_TYPE_TOOLS, RGN_TYPE_UI));  /* XXX, should use some better check? */
 	/* offset panels for small vertical tab area */
 	const char *category = NULL;
 	const int category_tabs_width = UI_PANEL_CATEGORY_MARGIN_WIDTH;
