@@ -225,7 +225,7 @@ static void file_refresh(const bContext *C, ScrArea *sa)
 	filelist_setsorting(sfile->files, params->sort);
 	filelist_setfilter_options(sfile->files, (params->flag & FILE_HIDE_DOT) != 0,
 	                                         false, /* TODO hide_parent, should be controllable? */
-	                                         params->flag & FILE_FILTER ? params->filter : 0,
+	                                         (params->flag & FILE_FILTER) ? params->filter : 0,
 	                                         params->filter_id,
 	                                         params->filter_glob,
 	                                         params->filter_search);
@@ -435,6 +435,7 @@ static void file_operatortypes(void)
 	WM_operatortype_append(FILE_OT_bookmark_cleanup);
 	WM_operatortype_append(FILE_OT_bookmark_move);
 	WM_operatortype_append(FILE_OT_reset_recent);
+	WM_operatortype_append(FILE_OT_hidedot);
 	WM_operatortype_append(FILE_OT_filenum);
 	WM_operatortype_append(FILE_OT_directory_new);
 	WM_operatortype_append(FILE_OT_delete);
