@@ -39,6 +39,8 @@
 
 #include "BKE_context.h"
 #include "BKE_global.h"
+#include "BKE_library.h"
+#include "BKE_main.h"
 #include "BKE_node.h"
 
 #include "ED_node.h"  /* own include */
@@ -59,6 +61,104 @@
 
 #include "node_intern.h"  /* own include */
 
+/* ****************** Relations helpers *********************** */
+
+static bool ntree_check_nodes_connected_dfs(bNodeTree *ntree,
+                                            bNode *from,
+                                            bNode *to)
+{
+	if (from->flag & NODE_TEST) {
+		return false;
+	}
+	from->flag |= NODE_TEST;
+	for (bNodeLink *link = ntree->links.first; link != NULL; link = link->next) {
+		if (link->fromnode == from) {
+			if (link->tonode == to) {
+				return true;
+			}
+			else {
+				if (ntree_check_nodes_connected_dfs(ntree, link->tonode, to)) {
+					return true;
+				}
+			}
+		}
+	}
+	return false;
+}
+
+static bool ntree_check_nodes_connected(bNodeTree *ntree, bNode *from, bNode *to)
+{
+	if (from == to) {
+		return true;
+	}
+	ntreeNodeFlagSet(ntree, NODE_TEST, false);
+	return ntree_check_nodes_connected_dfs(ntree, from, to);
+}
+
+static bool node_group_has_output_dfs(bNode *node)
+{
+	bNodeTree *ntree = (bNodeTree *)node->id;
+	if (ntree->id.tag & LIB_TAG_DOIT) {
+		return false;
+	}
+	ntree->id.tag |= LIB_TAG_DOIT;
+	for (bNode *current_node = ntree->nodes.first;
+	     current_node != NULL;
+	     current_node = current_node->next)
+	{
+		if (current_node->type == NODE_GROUP) {
+			if (current_node->id && node_group_has_output_dfs(current_node)) {
+				return true;
+			}
+		}
+		if (current_node->flag & NODE_DO_OUTPUT &&
+		    current_node->type != NODE_GROUP_OUTPUT)
+		{
+			return true;
+		}
+	}
+	return false;
+}
+
+static bool node_group_has_output(bNode *node)
+{
+	Main *bmain = G.main;
+	BLI_assert(node->type == NODE_GROUP);
+	bNodeTree *ntree = (bNodeTree *)node->id;
+	if (ntree == NULL) {
+		return false;
+	}
+	BKE_main_id_tag_listbase(&bmain->nodetree, LIB_TAG_DOIT, false);
+	return node_group_has_output_dfs(node);
+}
+
+bool node_connected_to_output(bNodeTree *ntree, bNode *node)
+{
+	for (bNode *current_node = ntree->nodes.first;
+	     current_node != NULL;
+	     current_node = current_node->next)
+	{
+		/* Special case for group nodes -- if modified node connected to a group
+		 * with active output inside we consider refresh is needed.
+		 *
+		 * We could make check more grained here by taking which socket the node
+		 * is connected to and so eventually.
+		 */
+		if (current_node->type == NODE_GROUP &&
+		    ntree_check_nodes_connected(ntree, node, current_node) &&
+		    node_group_has_output(current_node))
+		{
+			return true;
+		}
+		if (current_node->flag & NODE_DO_OUTPUT) {
+			if (ntree_check_nodes_connected(ntree, node, current_node)) {
+				return true;
+			}
+		}
+	}
+	return false;
+}
+
 /* ****************** Add *********************** */
 
 
@@ -176,25 +276,16 @@ static bNodeSocket *best_socket_input(bNodeTree *ntree, bNode *node, int num, in
 	return NULL;
 }
 
-static int snode_autoconnect_input(SpaceNode *snode, bNode *node_fr, bNodeSocket *sock_fr, bNode *node_to, bNodeSocket *sock_to, int replace)
+static bool snode_autoconnect_input(SpaceNode *snode, bNode *node_fr, bNodeSocket *sock_fr, bNode *node_to, bNodeSocket *sock_to, int replace)
 {
 	bNodeTree *ntree = snode->edittree;
-	bNodeLink *link;
 
 	/* then we can connect */
 	if (replace)
 		nodeRemSocketLinks(ntree, sock_to);
 
-	link = nodeAddLink(ntree, node_fr, sock_fr, node_to, sock_to);
-	/* validate the new link */
-	ntreeUpdateTree(G.main, ntree);
-	if (!(link->flag & NODE_LINK_VALID)) {
-		nodeRemLink(ntree, link);
-		return 0;
-	}
-
-	snode_update(snode, node_to);
-	return 1;
+	nodeAddLink(ntree, node_fr, sock_fr, node_to, sock_to);
+	return true;
 }
 
 static void snode_autoconnect(SpaceNode *snode, const bool allow_multiple, const bool replace)
@@ -225,6 +316,10 @@ static void snode_autoconnect(SpaceNode *snode, const bool allow_multiple, const
 
 		node_fr = nli->node;
 		node_to = nli->next->node;
+		/* corner case: input/output node aligned the wrong way around (T47729) */
+		if (BLI_listbase_is_empty(&node_to->inputs) || BLI_listbase_is_empty(&node_fr->outputs)) {
+			SWAP(bNode *, node_fr, node_to);
+		}
 
 		/* if there are selected sockets, connect those */
 		for (sock_to = node_to->inputs.first; sock_to; sock_to = sock_to->next) {
@@ -410,12 +505,10 @@ void NODE_OT_link_viewer(wmOperatorType *ot)
 
 static void node_link_update_header(bContext *C, bNodeLinkDrag *UNUSED(nldrag))
 {
-#define HEADER_LENGTH 256
-	char header[HEADER_LENGTH];
+	char header[UI_MAX_DRAW_STR];
 
-	BLI_strncpy(header, IFACE_("LMB: drag node link, RMB: cancel"), HEADER_LENGTH);
+	BLI_strncpy(header, IFACE_("LMB: drag node link, RMB: cancel"), sizeof(header));
 	ED_area_headerprint(CTX_wm_area(C), header);
-#undef HEADER_LENGTH
 }
 
 static int node_count_links(bNodeTree *ntree, bNodeSocket *sock)
@@ -468,7 +561,8 @@ static void node_link_exit(bContext *C, wmOperator *op, bool apply_links)
 	bNodeTree *ntree = snode->edittree;
 	bNodeLinkDrag *nldrag = op->customdata;
 	LinkData *linkdata;
-	
+	bool do_tag_update = false;
+
 	/* avoid updates while applying links */
 	ntree->is_updating = true;
 	for (linkdata = nldrag->links.first; linkdata; linkdata = linkdata->next) {
@@ -493,15 +587,27 @@ static void node_link_exit(bContext *C, wmOperator *op, bool apply_links)
 			
 			/* we might need to remove a link */
 			node_remove_extra_links(snode, link);
+
+			if (link->tonode) {
+				do_tag_update |= (do_tag_update || node_connected_to_output(ntree, link->tonode));
+			}
 		}
-		else
+		else {
+			/* See note below, but basically TEST flag means that the link
+			 * was connected to output (or to a node which affects the
+			 * output).
+			 */
+			do_tag_update |= (link->flag & NODE_LINK_TEST) != 0;
 			nodeRemLink(ntree, link);
+		}
 	}
 	ntree->is_updating = false;
 	
 	ntreeUpdateTree(CTX_data_main(C), ntree);
 	snode_notify(C, snode);
-	snode_dag_update(C, snode);
+	if (do_tag_update) {
+		snode_dag_update(C, snode);
+	}
 	
 	BLI_remlink(&snode->linkdrag, nldrag);
 	/* links->data pointers are either held by the tree or freed already */
@@ -632,7 +738,18 @@ static bNodeLinkDrag *node_link_init(SpaceNode *snode, float cursor[2], bool det
 					*oplink = *link;
 					oplink->next = oplink->prev = NULL;
 					oplink->flag |= NODE_LINK_VALID;
-					
+
+					/* The link could be disconnected and in that case we
+					 * wouldn't be able to check whether tag update is
+					 * needed or not when releasing mouse button. So we
+					 * cache whether the link affects output or not here
+					 * using TEST flag.
+					 */
+					oplink->flag &= ~NODE_LINK_TEST;
+					if (node_connected_to_output(snode->edittree, link->tonode)) {
+						oplink->flag |= NODE_LINK_TEST;
+					}
+
 					BLI_addtail(&nldrag->links, linkdata);
 					nodeRemLink(snode->edittree, link);
 				}
@@ -647,7 +764,11 @@ static bNodeLinkDrag *node_link_init(SpaceNode *snode, float cursor[2], bool det
 			oplink->fromnode = node;
 			oplink->fromsock = sock;
 			oplink->flag |= NODE_LINK_VALID;
-			
+			oplink->flag &= ~NODE_LINK_TEST;
+			if (node_connected_to_output(snode->edittree, node)) {
+				oplink->flag |= NODE_LINK_TEST;
+			}
+
 			BLI_addtail(&nldrag->links, linkdata);
 		}
 	}
@@ -668,7 +789,11 @@ static bNodeLinkDrag *node_link_init(SpaceNode *snode, float cursor[2], bool det
 					*oplink = *link;
 					oplink->next = oplink->prev = NULL;
 					oplink->flag |= NODE_LINK_VALID;
-					
+					oplink->flag &= ~NODE_LINK_TEST;
+					if (node_connected_to_output(snode->edittree, link->tonode)) {
+						oplink->flag |= NODE_LINK_TEST;
+					}
+
 					BLI_addtail(&nldrag->links, linkdata);
 					nodeRemLink(snode->edittree, link);
 					
@@ -687,7 +812,11 @@ static bNodeLinkDrag *node_link_init(SpaceNode *snode, float cursor[2], bool det
 			oplink->tonode = node;
 			oplink->tosock = sock;
 			oplink->flag |= NODE_LINK_VALID;
-			
+			oplink->flag &= ~NODE_LINK_TEST;
+			if (node_connected_to_output(snode->edittree, node)) {
+				oplink->flag |= NODE_LINK_TEST;
+			}
+
 			BLI_addtail(&nldrag->links, linkdata);
 		}
 	}
@@ -817,6 +946,7 @@ static int cut_links_exec(bContext *C, wmOperator *op)
 	ARegion *ar = CTX_wm_region(C);
 	float mcoords[256][2];
 	int i = 0;
+	bool do_tag_update = false;
 
 	RNA_BEGIN (op->ptr, itemptr, "path")
 	{
@@ -849,6 +979,8 @@ static int cut_links_exec(bContext *C, wmOperator *op)
 					found = true;
 				}
 
+				do_tag_update |= (do_tag_update || node_connected_to_output(snode->edittree, link->tonode));
+
 				snode_update(snode, link->tonode);
 				nodeRemLink(snode->edittree, link);
 			}
@@ -857,7 +989,9 @@ static int cut_links_exec(bContext *C, wmOperator *op)
 		if (found) {
 			ntreeUpdateTree(CTX_data_main(C), snode->edittree);
 			snode_notify(C, snode);
-			snode_dag_update(C, snode);
+			if (do_tag_update) {
+				snode_dag_update(C, snode);
+			}
 
 			return OPERATOR_FINISHED;
 		}
@@ -1583,7 +1717,7 @@ static int node_insert_offset_modal(bContext *C, wmOperator *UNUSED(op), const w
 	float duration;
 	bool redraw = false;
 
-	if (!snode || event->type != TIMER || iofsd->anim_timer != event->customdata)
+	if (!snode || event->type != TIMER || iofsd == NULL || iofsd->anim_timer != event->customdata)
 		return OPERATOR_PASS_THROUGH;
 
 	duration = (float)iofsd->anim_timer->duration;
