@@ -279,8 +279,6 @@ static void draw_uvs_stretch(SpaceImage *sima, Scene *scene, BMEditMesh *em, MTe
 
 			col[3] = 0.5f; /* hard coded alpha, not that nice */
 			
-			glShadeModel(GL_SMOOTH);
-			
 			BM_ITER_MESH (efa, &iter, bm, BM_FACES_OF_MESH) {
 				tf = BM_ELEM_CD_GET_VOID_P(efa, cd_poly_tex_offset);
 				
@@ -344,8 +342,6 @@ static void draw_uvs_stretch(SpaceImage *sima, Scene *scene, BMEditMesh *em, MTe
 			BLI_buffer_free(&av_buf);
 			BLI_buffer_free(&auv_buf);
 
-			glShadeModel(GL_FLAT);
-
 			break;
 		}
 	}
@@ -794,8 +790,7 @@ static void draw_uvs(SpaceImage *sima, Scene *scene, Object *obedit)
 				UI_GetThemeColor4ubv(TH_EDGE_SELECT, col1);
 
 				if (interpedges) {
-					glShadeModel(GL_SMOOTH);
-
+					GPU_basic_shader_bind(GPU_SHADER_USE_COLOR);
 					BM_ITER_MESH (efa, &iter, bm, BM_FACES_OF_MESH) {
 						if (!BM_elem_flag_test(efa, BM_ELEM_TAG))
 							continue;
@@ -810,8 +805,6 @@ static void draw_uvs(SpaceImage *sima, Scene *scene, Object *obedit)
 						}
 						glEnd();
 					}
-
-					glShadeModel(GL_FLAT);
 				}
 				else {
 					BM_ITER_MESH (efa, &iter, bm, BM_FACES_OF_MESH) {
