@@ -83,18 +83,10 @@
 #  include <libswscale/swscale.h>
 #endif
 
-/* actually hard coded endianness */
-#define GET_BIG_LONG(x) (((uchar *) (x))[0] << 24 | ((uchar *) (x))[1] << 16 | ((uchar *) (x))[2] << 8 | ((uchar *) (x))[3])
-#define GET_LITTLE_LONG(x) (((uchar *) (x))[3] << 24 | ((uchar *) (x))[2] << 16 | ((uchar *) (x))[1] << 8 | ((uchar *) (x))[0])
-#define SWAP_L(x) (((x << 24) & 0xff000000) | ((x << 8) & 0xff0000) | ((x >> 8) & 0xff00) | ((x >> 24) & 0xff))
-#define SWAP_S(x) (((x << 8) & 0xff00) | ((x >> 8) & 0xff))
-
 /* more endianness... should move to a separate file... */
 #ifdef __BIG_ENDIAN__
-#  define GET_ID GET_BIG_LONG
 #  define LITTLE_LONG SWAP_LONG
 #else
-#  define GET_ID GET_LITTLE_LONG
 #  define LITTLE_LONG ENDIAN_NOP
 #endif
 
