@@ -522,6 +522,10 @@ static void rna_def_render_engine(BlenderRNA *brna)
 	prop = RNA_def_boolean(func, "do_break", 0, "Break", "");
 	RNA_def_function_return(func, prop);
 
+	func = RNA_def_function(srna, "active_view_get", "RE_engine_active_view_get");
+	prop = RNA_def_string(func, "view", NULL, 0, "View", "Single view active");
+	RNA_def_function_return(func, prop);
+
 	func = RNA_def_function(srna, "active_view_set", "RE_engine_active_view_set");
 	RNA_def_string(func, "view", NULL, 0, "View", "Single view to set as active");  /* NULL ok here */
 	RNA_def_property_flag(prop, PROP_REQUIRED);
@@ -529,15 +533,23 @@ static void rna_def_render_engine(BlenderRNA *brna)
 	func = RNA_def_function(srna, "camera_shift_x", "RE_engine_get_camera_shift_x");
 	prop = RNA_def_pointer(func, "camera", "Object", "", "");
 	RNA_def_property_flag(prop, PROP_REQUIRED);
+	prop = RNA_def_boolean(func, "use_spherical_stereo", 0, "Spherical Stereo", "");
 	prop = RNA_def_float(func, "shift_x", 0.0f, 0.0f, FLT_MAX, "Shift X", "", 0.0f, FLT_MAX);
 	RNA_def_function_return(func, prop);
 
 	func = RNA_def_function(srna, "camera_model_matrix", "RE_engine_get_camera_model_matrix");
 	prop = RNA_def_pointer(func, "camera", "Object", "", "");
 	RNA_def_property_flag(prop, PROP_REQUIRED);
+	prop = RNA_def_boolean(func, "use_spherical_stereo", 0, "Spherical Stereo", "");
 	prop = RNA_def_float_matrix(func, "r_model_matrix", 4, 4, NULL, 0.0f, 0.0f, "Model Matrix", "Normalized camera model matrix", 0.0f, 0.0f);
 	RNA_def_property_flag(prop, PROP_REQUIRED);
 
+	func = RNA_def_function(srna, "use_spherical_stereo", "RE_engine_get_spherical_stereo");
+	prop = RNA_def_pointer(func, "camera", "Object", "", "");
+	RNA_def_property_flag(prop, PROP_REQUIRED);
+	prop = RNA_def_boolean(func, "use_spherical_stereo", 0, "Spherical Stereo", "");
+	RNA_def_function_return(func, prop);
+
 	func = RNA_def_function(srna, "update_stats", "RE_engine_update_stats");
 	RNA_def_function_ui_description(func, "Update and signal to redraw render status text");
 	prop = RNA_def_string(func, "stats", NULL, 0, "Stats", "");
@@ -667,6 +679,10 @@ static void rna_def_render_engine(BlenderRNA *brna)
 	RNA_def_property_boolean_sdna(prop, NULL, "type->flag", RE_USE_SAVE_BUFFERS);
 	RNA_def_property_flag(prop, PROP_REGISTER_OPTIONAL);
 
+	prop = RNA_def_property(srna, "bl_use_spherical_stereo", PROP_BOOLEAN, PROP_NONE);
+	RNA_def_property_boolean_sdna(prop, NULL, "type->flag", RE_USE_SPHERICAL_STEREO);
+	RNA_def_property_flag(prop, PROP_REGISTER_OPTIONAL);
+
 	RNA_define_verify_sdna(1);
 }
 
