@@ -64,12 +64,19 @@ static void node_shader_init_tex_brick(bNodeTree *UNUSED(ntree), bNode *node)
 
 static int node_shader_gpu_tex_brick(GPUMaterial *mat, bNode *node, bNodeExecData *UNUSED(execdata), GPUNodeStack *in, GPUNodeStack *out)
 {
-	if (!in[0].link)
+	if (!in[0].link) {
 		in[0].link = GPU_attribute(CD_ORCO, "");
+		GPU_link(mat, "generated_from_orco", in[0].link, &in[0].link);
+	}
 
 	node_shader_gpu_tex_mapping(mat, node, in, out);
-
-	return GPU_stack_link(mat, "node_tex_brick", in, out);
+	NodeTexBrick *tex = (NodeTexBrick *)node->storage;
+	float offset_freq = tex->offset_freq;
+	float squash_freq = tex->squash_freq;
+	return GPU_stack_link(mat, "node_tex_brick",
+	                      in, out,
+	                      GPU_uniform(&tex->offset), GPU_uniform(&offset_freq),
+	                      GPU_uniform(&tex->squash), GPU_uniform(&squash_freq));
 }
 
 /* node type definition */
