@@ -42,6 +42,8 @@
 
 #include "BLI_math.h"
 #include "BLI_utildefines.h"
+
+#include "../generic/py_capi_utils.h"
 #include "../generic/python_utildefines.h"
 
 /*-------------------------DOC STRINGS ---------------------------*/
@@ -77,12 +79,14 @@ static PyObject *M_Geometry_intersect_ray_tri(PyObject *UNUSED(self), PyObject *
 	PyObject *py_ray, *py_ray_off, *py_tri[3];
 	float dir[3], orig[3], tri[3][3], e1[3], e2[3], pvec[3], tvec[3], qvec[3];
 	float det, inv_det, u, v, t;
-	int clip = 1;
+	bool clip = true;
 	int i;
 
 	if (!PyArg_ParseTuple(
-	        args, "OOOOO|i:intersect_ray_tri",
-	        UNPACK3_EX(&, py_tri, ), &py_ray, &py_ray_off, &clip))
+	        args, "OOOOO|O&:intersect_ray_tri",
+	        UNPACK3_EX(&, py_tri, ),
+	        &py_ray, &py_ray_off,
+	        PyC_ParseBool, &clip))
 	{
 		return NULL;
 	}
@@ -199,7 +203,7 @@ static PyObject *M_Geometry_intersect_line_line(PyObject *UNUSED(self), PyObject
 	}
 
 	if (result == 0) {
-		/* colinear */
+		/* collinear */
 		Py_RETURN_NONE;
 	}
 	else {
@@ -411,7 +415,9 @@ static PyObject *M_Geometry_volume_tetrahedron(PyObject *UNUSED(self), PyObject
 PyDoc_STRVAR(M_Geometry_intersect_line_line_2d_doc,
 ".. function:: intersect_line_line_2d(lineA_p1, lineA_p2, lineB_p1, lineB_p2)\n"
 "\n"
-"   Takes 2 lines (as 4 vectors) and returns a vector for their point of intersection or None.\n"
+"   Takes 2 segments (defined by 4 vectors) and returns a vector for their point of intersection or None.\n"
+"\n"
+"   .. warning:: Despite its name, this function works on segments, and not on lines.\n"
 "\n"
 "   :arg lineA_p1: First point of the first line\n"
 "   :type lineA_p1: :class:`mathutils.Vector`\n"
@@ -477,12 +483,12 @@ static PyObject *M_Geometry_intersect_line_plane(PyObject *UNUSED(self), PyObjec
 	PyObject *py_line_a, *py_line_b, *py_plane_co, *py_plane_no;
 	float line_a[3], line_b[3], plane_co[3], plane_no[3];
 	float isect[3];
-	int no_flip = false;
+	bool no_flip = false;
 
 	if (!PyArg_ParseTuple(
-	        args, "OOOO|i:intersect_line_plane",
+	        args, "OOOO|O&:intersect_line_plane",
 	        &py_line_a, &py_line_b, &py_plane_co, &py_plane_no,
-	        &no_flip))
+	        PyC_ParseBool, &no_flip))
 	{
 		return NULL;
 	}
@@ -526,6 +532,7 @@ static PyObject *M_Geometry_intersect_plane_plane(PyObject *UNUSED(self), PyObje
 	PyObject *ret, *ret_co, *ret_no;
 	PyObject *py_plane_a_co, *py_plane_a_no, *py_plane_b_co, *py_plane_b_no;
 	float plane_a_co[3], plane_a_no[3], plane_b_co[3], plane_b_no[3];
+	float plane_a[4], plane_b[4];
 
 	float isect_co[3];
 	float isect_no[3];
@@ -545,9 +552,12 @@ static PyObject *M_Geometry_intersect_plane_plane(PyObject *UNUSED(self), PyObje
 		return NULL;
 	}
 
-	if (isect_plane_plane_v3(isect_co, isect_no,
-	                         plane_a_co, plane_a_no,
-	                         plane_b_co, plane_b_no))
+	plane_from_point_normal_v3(plane_a, plane_a_co, plane_a_no);
+	plane_from_point_normal_v3(plane_b, plane_b_co, plane_b_no);
+
+	if (isect_plane_plane_v3(
+	        plane_a, plane_b,
+	        isect_co, isect_no))
 	{
 		normalize_v3(isect_no);
 
@@ -589,14 +599,15 @@ static PyObject *M_Geometry_intersect_line_sphere(PyObject *UNUSED(self), PyObje
 	PyObject *py_line_a, *py_line_b, *py_sphere_co;
 	float line_a[3], line_b[3], sphere_co[3];
 	float sphere_radius;
-	int clip = true;
+	bool clip = true;
 
 	float isect_a[3];
 	float isect_b[3];
 
 	if (!PyArg_ParseTuple(
-	        args, "OOOf|i:intersect_line_sphere",
-	        &py_line_a, &py_line_b, &py_sphere_co, &sphere_radius, &clip))
+	        args, "OOOf|O&:intersect_line_sphere",
+	        &py_line_a, &py_line_b, &py_sphere_co, &sphere_radius,
+	        PyC_ParseBool, &clip))
 	{
 		return NULL;
 	}
@@ -661,14 +672,15 @@ static PyObject *M_Geometry_intersect_line_sphere_2d(PyObject *UNUSED(self), PyO
 	PyObject *py_line_a, *py_line_b, *py_sphere_co;
 	float line_a[2], line_b[2], sphere_co[2];
 	float sphere_radius;
-	int clip = true;
+	bool clip = true;
 
 	float isect_a[2];
 	float isect_b[2];
 
 	if (!PyArg_ParseTuple(
-	        args, "OOOf|i:intersect_line_sphere_2d",
-	        &py_line_a, &py_line_b, &py_sphere_co, &sphere_radius, &clip))
+	        args, "OOOf|O&:intersect_line_sphere_2d",
+	        &py_line_a, &py_line_b, &py_sphere_co, &sphere_radius,
+	        PyC_ParseBool, &clip))
 	{
 		return NULL;
 	}
@@ -850,7 +862,7 @@ PyDoc_STRVAR(M_Geometry_intersect_point_quad_2d_doc,
 "\n"
 "   Takes 5 vectors (using only the x and y coordinates): one is the point and the next 4 define the quad, \n"
 "   only the x and y are used from the vectors. Returns 1 if the point is within the quad, otherwise 0.\n"
-"   Works only with convex quads without singular edges."
+"   Works only with convex quads without singular edges.\n"
 "\n"
 "   :arg pt: Point\n"
 "   :type pt: :class:`mathutils.Vector`\n"
@@ -860,7 +872,7 @@ PyDoc_STRVAR(M_Geometry_intersect_point_quad_2d_doc,
 "   :type quad_p2: :class:`mathutils.Vector`\n"
 "   :arg quad_p3: Third point of the quad\n"
 "   :type quad_p3: :class:`mathutils.Vector`\n"
-"   :arg quad_p4: Forth point of the quad\n"
+"   :arg quad_p4: Fourth point of the quad\n"
 "   :type quad_p4: :class:`mathutils.Vector`\n"
 "   :rtype: int\n"
 );
