@@ -53,6 +53,7 @@
 #include "ED_screen.h"
 
 #include "GPU_glew.h"
+#include "GPU_basic_shader.h"
 
 #include "WM_api.h"
 #include "WM_types.h"
@@ -82,76 +83,48 @@ static void wm_method_draw_stereo3d_pageflip(wmWindow *win)
 	glDrawBuffer(GL_BACK);
 }
 
-static GLuint left_interlace_mask[32];
-static GLuint right_interlace_mask[32];
 static enum eStereo3dInterlaceType interlace_prev_type = -1;
 static char interlace_prev_swap = -1;
 
-static void wm_interlace_masks_create(wmWindow *win)
-{
-	GLuint pattern;
-	char i;
-	bool swap = (win->stereo3d_format->flag & S3D_INTERLACE_SWAP) != 0;
-	enum eStereo3dInterlaceType interlace_type = win->stereo3d_format->interlace_type;
-
-	if (interlace_prev_type == interlace_type && interlace_prev_swap == swap)
-		return;
-
-	switch (interlace_type) {
-		case S3D_INTERLACE_ROW:
-			pattern = 0x00000000;
-			pattern = swap ? ~pattern : pattern;
-			for (i = 0; i < 32; i += 2) {
-				left_interlace_mask[i] = pattern;
-				right_interlace_mask[i] = ~pattern;
-			}
-			for (i = 1; i < 32; i += 2) {
-				left_interlace_mask[i] = ~pattern;
-				right_interlace_mask[i] = pattern;
-			}
-			break;
-		case S3D_INTERLACE_COLUMN:
-			pattern = 0x55555555;
-			pattern = swap ? ~pattern : pattern;
-			for (i = 0; i < 32; i++) {
-				left_interlace_mask[i] = pattern;
-				right_interlace_mask[i] = ~pattern;
-			}
-			break;
-		case S3D_INTERLACE_CHECKERBOARD:
-		default:
-			pattern = 0x55555555;
-			pattern = swap ? ~pattern : pattern;
-			for (i = 0; i < 32; i += 2) {
-				left_interlace_mask[i] = pattern;
-				right_interlace_mask[i] = ~pattern;
-			}
-			for (i = 1; i < 32; i += 2) {
-				left_interlace_mask[i] = ~pattern;
-				right_interlace_mask[i] = pattern;
-			}
-			break;
-	}
-	interlace_prev_type = interlace_type;
-	interlace_prev_swap = swap;
-}
-
 static void wm_method_draw_stereo3d_interlace(wmWindow *win)
 {
 	wmDrawData *drawdata;
 	int view;
-
-	wm_interlace_masks_create(win);
+	bool flag;
+	bool swap = (win->stereo3d_format->flag & S3D_INTERLACE_SWAP) != 0;
+	enum eStereo3dInterlaceType interlace_type = win->stereo3d_format->interlace_type;
 
 	for (view = 0; view < 2; view ++) {
+		flag = swap ? !view : view;
 		drawdata = BLI_findlink(&win->drawdata, (view * 2) + 1);
-
-		glEnable(GL_POLYGON_STIPPLE);
-		glPolygonStipple(view ? (GLubyte *) right_interlace_mask : (GLubyte *) left_interlace_mask);
+		GPU_basic_shader_bind(GPU_SHADER_STIPPLE);
+		switch (interlace_type) {
+			case S3D_INTERLACE_ROW:
+				if (flag)
+					GPU_basic_shader_stipple(GPU_SHADER_STIPPLE_S3D_INTERLACE_ROW_SWAP);
+				else
+					GPU_basic_shader_stipple(GPU_SHADER_STIPPLE_S3D_INTERLACE_ROW);
+				break;
+			case S3D_INTERLACE_COLUMN:
+				if (flag)
+					GPU_basic_shader_stipple(GPU_SHADER_STIPPLE_S3D_INTERLACE_COLUMN_SWAP);
+				else
+					GPU_basic_shader_stipple(GPU_SHADER_STIPPLE_S3D_INTERLACE_COLUMN);
+				break;
+			case S3D_INTERLACE_CHECKERBOARD:
+			default:
+				if (flag)
+					GPU_basic_shader_stipple(GPU_SHADER_STIPPLE_S3D_INTERLACE_CHECKER_SWAP);
+				else
+					GPU_basic_shader_stipple(GPU_SHADER_STIPPLE_S3D_INTERLACE_CHECKER);
+				break;
+		}
 
 		wm_triple_draw_textures(win, drawdata->triple, 1.0f);
-		glDisable(GL_POLYGON_STIPPLE);
+		GPU_basic_shader_bind(GPU_SHADER_USE_COLOR);
 	}
+	interlace_prev_type = interlace_type;
+	interlace_prev_swap = swap;
 }
 
 static void wm_method_draw_stereo3d_anaglyph(wmWindow *win)
@@ -195,7 +168,6 @@ static void wm_method_draw_stereo3d_sidebyside(wmWindow *win)
 	wmDrawData *drawdata;
 	wmDrawTriple *triple;
 	float halfx, halfy, ratiox, ratioy;
-	int x, y, offx, offy;
 	float alpha = 1.0f;
 	int view;
 	int soffx;
@@ -217,44 +189,40 @@ static void wm_method_draw_stereo3d_sidebyside(wmWindow *win)
 
 		glEnable(triple->target);
 
-		for (y = 0, offy = 0; y < triple->ny; offy += triple->y[y], y++) {
-			for (x = 0, offx = 0; x < triple->nx; offx += triple->x[x], x++) {
-				const int sizex = triple->x[x];
-				const int sizey = triple->y[y];
+		const int sizex = triple->x;
+		const int sizey = triple->y;
 
-				/* wmOrtho for the screen has this same offset */
-				ratiox = sizex;
-				ratioy = sizey;
-				halfx = GLA_PIXEL_OFS;
-				halfy = GLA_PIXEL_OFS;
+		/* wmOrtho for the screen has this same offset */
+		ratiox = sizex;
+		ratioy = sizey;
+		halfx = GLA_PIXEL_OFS;
+		halfy = GLA_PIXEL_OFS;
 
-				/* texture rectangle has unnormalized coordinates */
-				if (triple->target == GL_TEXTURE_2D) {
-					ratiox /= triple->x[x];
-					ratioy /= triple->y[y];
-					halfx /= triple->x[x];
-					halfy /= triple->y[y];
-				}
-
-				glBindTexture(triple->target, triple->bind[x + y * triple->nx]);
-
-				glColor4f(1.0f, 1.0f, 1.0f, alpha);
-				glBegin(GL_QUADS);
-				glTexCoord2f(halfx, halfy);
-				glVertex2f(soffx + (offx * 0.5f), offy);
-
-				glTexCoord2f(ratiox + halfx, halfy);
-				glVertex2f(soffx + ((offx + sizex) * 0.5f), offy);
-
-				glTexCoord2f(ratiox + halfx, ratioy + halfy);
-				glVertex2f(soffx + ((offx + sizex) * 0.5f), offy + sizey);
-
-				glTexCoord2f(halfx, ratioy + halfy);
-				glVertex2f(soffx + (offx * 0.5f), offy + sizey);
-				glEnd();
-			}
+		/* texture rectangle has unnormalized coordinates */
+		if (triple->target == GL_TEXTURE_2D) {
+			ratiox /= triple->x;
+			ratioy /= triple->y;
+			halfx /= triple->x;
+			halfy /= triple->y;
 		}
 
+		glBindTexture(triple->target, triple->bind);
+
+		glColor4f(1.0f, 1.0f, 1.0f, alpha);
+		glBegin(GL_QUADS);
+		glTexCoord2f(halfx, halfy);
+		glVertex2f(soffx, 0);
+
+		glTexCoord2f(ratiox + halfx, halfy);
+		glVertex2f(soffx + (sizex * 0.5f), 0);
+
+		glTexCoord2f(ratiox + halfx, ratioy + halfy);
+		glVertex2f(soffx + (sizex * 0.5f), sizey);
+
+		glTexCoord2f(halfx, ratioy + halfy);
+		glVertex2f(soffx, sizey);
+		glEnd();
+
 		glBindTexture(triple->target, 0);
 		glDisable(triple->target);
 		glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
@@ -266,7 +234,6 @@ static void wm_method_draw_stereo3d_topbottom(wmWindow *win)
 	wmDrawData *drawdata;
 	wmDrawTriple *triple;
 	float halfx, halfy, ratiox, ratioy;
-	int x, y, offx, offy;
 	float alpha = 1.0f;
 	int view;
 	int soffy;
@@ -284,44 +251,40 @@ static void wm_method_draw_stereo3d_topbottom(wmWindow *win)
 
 		glEnable(triple->target);
 
-		for (y = 0, offy = 0; y < triple->ny; offy += triple->y[y], y++) {
-			for (x = 0, offx = 0; x < triple->nx; offx += triple->x[x], x++) {
-				const int sizex = triple->x[x];
-				const int sizey = triple->y[y];
+		const int sizex = triple->x;
+		const int sizey = triple->y;
 
-				/* wmOrtho for the screen has this same offset */
-				ratiox = sizex;
-				ratioy = sizey;
-				halfx = GLA_PIXEL_OFS;
-				halfy = GLA_PIXEL_OFS;
+		/* wmOrtho for the screen has this same offset */
+		ratiox = sizex;
+		ratioy = sizey;
+		halfx = GLA_PIXEL_OFS;
+		halfy = GLA_PIXEL_OFS;
 
-				/* texture rectangle has unnormalized coordinates */
-				if (triple->target == GL_TEXTURE_2D) {
-					ratiox /= triple->x[x];
-					ratioy /= triple->y[y];
-					halfx /= triple->x[x];
-					halfy /= triple->y[y];
-				}
-
-				glBindTexture(triple->target, triple->bind[x + y * triple->nx]);
-
-				glColor4f(1.0f, 1.0f, 1.0f, alpha);
-				glBegin(GL_QUADS);
-				glTexCoord2f(halfx, halfy);
-				glVertex2f(offx, soffy + (offy * 0.5f));
-
-				glTexCoord2f(ratiox + halfx, halfy);
-				glVertex2f(offx + sizex, soffy + (offy * 0.5f));
-
-				glTexCoord2f(ratiox + halfx, ratioy + halfy);
-				glVertex2f(offx + sizex, soffy + ((offy + sizey) * 0.5f));
-
-				glTexCoord2f(halfx, ratioy + halfy);
-				glVertex2f(offx, soffy + ((offy + sizey) * 0.5f));
-				glEnd();
-			}
+		/* texture rectangle has unnormalized coordinates */
+		if (triple->target == GL_TEXTURE_2D) {
+			ratiox /= triple->x;
+			ratioy /= triple->y;
+			halfx /= triple->x;
+			halfy /= triple->y;
 		}
 
+		glBindTexture(triple->target, triple->bind);
+
+		glColor4f(1.0f, 1.0f, 1.0f, alpha);
+		glBegin(GL_QUADS);
+		glTexCoord2f(halfx, halfy);
+		glVertex2f(0, soffy);
+
+		glTexCoord2f(ratiox + halfx, halfy);
+		glVertex2f(sizex, soffy);
+
+		glTexCoord2f(ratiox + halfx, ratioy + halfy);
+		glVertex2f(sizex, soffy + (sizey * 0.5f));
+
+		glTexCoord2f(halfx, ratioy + halfy);
+		glVertex2f(0, soffy + (sizey * 0.5f));
+		glEnd();
+
 		glBindTexture(triple->target, 0);
 		glDisable(triple->target);
 		glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
