@@ -393,9 +393,10 @@ static PyObject *Texture_refresh(Texture *self, PyObject *args)
 					}
 					// load texture for rendering
 					loadTexture(self->m_actTex, texture, size, self->m_mipmap);
-
-					// refresh texture source, if required
-					if (refreshSource) self->m_source->m_image->refresh();
+				}
+				// refresh texture source, if required
+				if (refreshSource) {
+					self->m_source->m_image->refresh();
 				}
 			}
 		}
